## Automatically adapted for numpy.oldnumeric Jul 23, 2007 by 

#
# Last modified on Mon Mar 15 17:47:37 PST 2004 by lindy
#
# $Id: cluster.py,v 1.41 2010/06/21 15:47:17 rhuey Exp $
#
"""
Some definitions:
    cluster:    a list of conformations all within tolerance RMSD.
    clustering: a list of clusters from clustering a set of conformations
                at a given tolerance RMSD.
    Clusterer:  instances of this class perform the clustering operation,
                maintain the distance_matrix, and keep the dictionary of
                clusterings (keyed by tolerance).
"""

import string
import sys
import numpy.oldnumeric as Numeric
import UserList

class Cluster(UserList.UserList):
    """A cluster is a list of conformations.

    The 'rank' of a conformation is its order in the list. 
    """
    def __init__(self, seed, info=None):
        UserList.UserList.__init__(self, [seed])
        self.seed = seed
        if info:
            self.build(info)


    def build(self, info):
        #info is a dictionary 
        #confs is a ordered list of Conformations
        self.confs = info['confs']
        self.do_stats(info['min_energy'],
                      info['max_energy'],
                       info['average_energy'])
        self.rank = info['rank']
   

    def do_stats(self, min_energy=None, max_energy=None, avg_energy=None):
        # set me
        self.min_energy = min_energy
        self.max_energy = max_energy
        self.avg_energy = avg_energy



class Clustering(UserList.UserList):
    """A clustering is list of clusters resulting from a clustering operation.
    """
    def __init__(self):
        UserList.UserList.__init__(self)
        self.tolerance = None # set by the Clusterer


    def do_stats(self):
        """Compute some statistics about this clustering.
        """
        self.avg_size = Numeric.sum(map(len, self))/float(len(self))
        for c in self:
            c.do_stats()


        
class Clusterer:
    """This class knows how to cluster a list of conformations
    the Autodock way. That is sort conformations by energy, visit
    every conformation in order adding to the cluster if within the
    tolerance RMSD. Seed a new cluster other wise.

    self.data: the list of conformations to cluster
    self.clustering: a float-keyed dictionary. Keys are clustering
    tolerances (in angstroms).
    
    The list of Cluster instances is self.clusters
    """
    def __init__(self, data, sort='binding'):
        """data is a list of Conformations.
        """
        self.data = data
        # extract the binding (default) or docking energy
        if sort == 'docking':
            energy_list = [d.docking_energy for d in data]
        elif sort == 'intermolecular':
            energy_list = [d.intermol_energy for d in data]
        elif sort == 'flexres':
            if hasattr(data[0], 'subset'):
                ind = len(data[0].getCoords()) - len(data[0].subset) 
            else:
                print "no subsets have been set up for conformations!"
                return "ERROR"
            for d in data:
                d.flexres_energy = Numeric.add.reduce(d.total_energies[ind:])
                d.flexres_index = ind
            energy_list = [d.flexres_energy for d in data]
        else:
            energy_list = [d.binding_energy for d in data]

        # sort the conformations by energy
        self.argsort = Numeric.argsort(energy_list)
        self.energy_used = sort

        # save the pair-wise distances for reuse by get_distance
        self.dist_matrix = Numeric.zeros([len(data), len(data)]) - 1.0
        # set the customizable get_distance method to default
        self.set_get_distance(self._get_distance_default)
        
        self.clustering_dict = {}


    def _get_distance_flexres(self, a, b):
        """return RMSD between two conformations, a and b.
        """
        ax = self.data.index(a)
        bx = self.data.index(b)
        assert a.flexres_index == b.flexres_index
        ind = a.flexres_index

        if self.dist_matrix[ax][bx] >= 0.0:
            # return previously saved distance
            return self.dist_matrix[ax][bx]
        else:
            # compute, save, and return distance
            dist = a.getRMSD_subset(b.getCoords()[b.flexres_index:])
            self.dist_matrix[ax][bx] = self.dist_matrix[bx][ax] = dist
            return dist


    def _get_distance_custom(self, a, b):
        """return RMSD between two conformations, a and b.
        """
        ax = self.data.index(a)
        bx = self.data.index(b)

        if self.dist_matrix[ax][bx] >= 0.0:
            # return previously saved distance
            return self.dist_matrix[ax][bx]
        else:
            # compute, save, and return distance
            dist = a.getRMSD_custom(b.getCoords())
            self.dist_matrix[ax][bx] = self.dist_matrix[bx][ax] = dist
            return dist


    def _get_distance_default(self, a, b):
        """return RMSD between two conformations, a and b.
        """
        ax = self.data.index(a)
        bx = self.data.index(b)

        if self.dist_matrix[ax][bx] >= 0.0:
            # return previously saved distance
            return self.dist_matrix[ax][bx]
        else:
            # compute, save, and return distance
            dist = a.getRMSD(b.getCoords())
            self.dist_matrix[ax][bx] = self.dist_matrix[bx][ax] = dist
            return dist


    def _get_distance_subset(self, a, b):
        """return RMSD between subsets of atoms in two conformations, a and b.
        """
        ax = self.data.index(a)
        bx = self.data.index(b)

        if self.dist_matrix[ax][bx] >= 0.0:
            # return previously saved distance
            return self.dist_matrix[ax][bx]
        else:
            ind = len(a.getCoords()) - len(a.subset)
            # compute, save, and return distance
            dist = a.getRMSD_subset(b.getCoords()[ind:])
            self.dist_matrix[ax][bx] = self.dist_matrix[bx][ax] = dist
            return dist


    def set_get_distance(self, f):
        self.get_distance = f


    def get_distance(self, a, b):
        return self.get_distance


    def make_clustering(self, tolerance, ref=None):
        """
        tolerance is the RMSD tolerance in angstroms
        ref is the reference Conformation
        """
        this_clustering = Clustering()
        # seed the first cluster with supplied ref
        # or the lowest energy conformation in the data (as ref)
        if not ref:
            ref = self.data[int(self.argsort[0])]
        this_clustering.append( Cluster(ref))

        # go through all conformations
        for i in range(len(self.argsort)):
            conf = self.data[int(self.argsort[i])]
            if conf == ref: continue  # seen it before so skip it

            # try to find a home for this conf in all the this_clustering
            clustered = 0
            for cluster in this_clustering:
                # get dist from cluster seed to conformaton
                dist = self.get_distance(cluster[0], conf)

                if dist <= tolerance:
                    # then add to current cluster and stop looking
                    cluster.append(conf)
                    clustered = 1
                    break
            if not clustered:
                # then start new cluster
                this_clustering.append( Cluster(conf))

        # save the clustering in the dictionary
        self.clustering_dict[tolerance] = this_clustering
        this_clustering.tolerance = tolerance

        # tell each conformation its cluster and rank (for this tolerance)
        for cx, cluster in enumerate(this_clustering):
            for rank, conf in enumerate(cluster):
                conf.cluster_dict[tolerance] = (cx, rank)


    def get_max_dist(self, clust1, clust2):
        max_dist = -1.
        for conf_i in clust1:
            for conf_j in clust2:
                dist_ij = self.get_distance(conf_i, conf_j)
                if dist_ij>max_dist: max_dist = dist_ij
        return max_dist


    def get_min_dist(self, clust1, clust2):
        min_dist = 1000000000.
        for conf_i in clust1:
            for conf_j in clust2:
                dist_ij = self.get_distance(conf_i, conf_j)
                if dist_ij<min_dist: min_dist = dist_ij
        return min_dist


    def make_hierarchical_clustering(self, linkage='single', precision=1, debug=False):
        """ uses HierarchicalClustering of python-cluster as explained in 
        http://www.elet.polimi.it/upload/matteucc/Clustering/tutorial_html/hierarchical.html
        Available linkage choices are: 'single','complete', 'average' or 'uclus'"""
        try:
            from python_cluster.cluster import HierarchicalClustering
        except:
            print "HierarchicalClustering from python-cluster.cluster not found"
            return
        assert linkage in ['single', 'complete', 'average', 'uclus']
        cl = HierarchicalClustering(self.data, self.get_distance, linkage=linkage)
        #make sure all the distances have been calculated
        num_confs = len(self.data)
        for i in range(num_confs):
            for j in range(num_confs):
                self.get_distance(self.data[i], self.data[j])
        all_dists = self.dist_matrix.ravel().tolist()
        all_dists.sort()
        #remove the zeros from the diagonal
        dists = all_dists[num_confs:] 
        dists.sort()
        rmsds = []
        clustD = self.clustering_dict
        duplicates = []
        #precision parameter was added to try to permit
        #the user to control how many levels were calculated...
        # by default, round rmsd values to 1 decimal place
        # eg 0.1, 0.2 etc...
        #@@ need to test with data for which this might matter
        # with ind.dlg, it doesn't seem to make any difference...
        for i in dists:
            v = round(i, precision)
            if v not in rmsds:
                rmsds.append(v)
        for i in range(len(rmsds)):            
            level = rmsds[i]
            if debug: print "getlevel level=", level
            clustD[level] = cl.getlevel(level)
            if debug: print "built->len(clustD[",level,"]=", len(clustD[level])
            if len(clustD[level])==1:
                if debug: print "all in a single cluster"
  

    def show_clustering(self):
        dk = self.clustering_dict.keys()
        dk.sort()
        for val in dk:
            print val, ':'
            for clust in self.clustering_dict[val]:
                for conf in clust:
                    print conf.run,
                print
            print        
                    
    def write_summary(self, filename=None):
        """
        """
        if filename:
            file_ptr = open( filename, 'w')
        else:
            file_ptr = sys.stdout

        # make sure the keys happen in the same order
        sorted_keys = self.clustering_dict.keys()[:]
        sorted_keys.sort()
        # write tolerances
        for tolerance in sorted_keys:
            clustering = self.clustering_dict[tolerance]
            clustering.do_stats()
            file_ptr.write("%s  %3d  %3.2f" % (str(tolerance),
                                           len(clustering),
                                           clustering.avg_size))
            file_ptr.write("\n")


    def write(self, filename=None):
        """Write a set of clusterings to a file.

The clusterings are written to the filename given in
the following format:
1st line: space separated floats describing tolerances.
the rest: one line foreach conformation in order of
          increasing binding_energy. each line consiststs
          of space separated integers, two per clustering,
          the first gives the cluster index, the second
          gives the rank within the cluster.

For example:
1.0 2.0 3.0
0 0 0 0 0 0
0 1 0 2 0 0

New parameter for docking/binding energy denotes which was
used to create clusterings:
For example:
1.0 2.0 3.0 docking  -- specifies that DOCKING energy was used
1.0 2.0 3.0 binding  -- specifies that BINDING energy was used
1.0 2.0 3.0          -- if not specified then BINDING energy assumed.

Mon Mar 15 16:09:51 PST 2004
New fields reporting the rmsd values from the reference strucuture
(or lowest energy cluster seed if no reference structure given) and
rmsd values to each of the cluster seeds. Finally the energy of the
conformations is given. This is the docking energy or binding energy
as specified in the header. (This energy is for convenience as it is
just a copy of the energy from the AutoDock output.

For example:
1.5 2.0 binding
  0   0   0   0  0.000  0.000  0.000 -14.645
  1   0   1   0  2.449  0.000  0.000 -14.636
  0   1   0   1  1.281  1.281  1.281 -14.424
  1   1   1   1  2.548  1.014  1.014 -14.210
                   |      |      |
                   |      |      |-rmsd from cluster seed @ 2.0 tolerance
                   |      |-rmsd from cluster seed @ 1.0 tolerance
                   |-rmsd from overall reference structure      

The first line says there were three clusterings at 1.5 and 2.0 Angstrom
rmsd tolerances using the binding energy as the sort. The second conformation
started a new cluster because it was 2.449 A-rmsd from the the reference.
        """
        if filename:
            file_ptr = open( filename, 'w')
        else:
            file_ptr = sys.stdout

        # make sure the keys happen in the same order
        sorted_keys = self.clustering_dict.keys()[:]
        sorted_keys.sort()
        # write tolerances
        for key in sorted_keys:
            file_ptr.write("%6s  " % str(key))
        # note which energy was used to cluster
        file_ptr.write(self.energy_used)
        file_ptr.write("\n")

        # write one line per conformation
        for i in range(len(self.argsort)):
            conf = self.data[int(self.argsort[i])]
            # write the pair for each tolerance key
            for key in sorted_keys:
                file_ptr.write("%3d %3d " % conf.cluster_dict[key])
                
            # append the rmsd from the overall reference conf (or seed)
            file_ptr.write("%6.3f " % self.get_distance(
                self.clustering_dict[key][0][0],
                conf))
            # append the rmsd from the clustering seeds
            for key in sorted_keys:
                file_ptr.write("%6.3f " % self.get_distance(
                    self.clustering_dict[key][conf.cluster_dict[key][0]][0],
                    conf))
            # and finally the appropriate energy
            if self.energy_used=='energy':
                file_ptr.write("%7.3f " %
                           getattr(conf, "binding_energy" ))
            else:
                file_ptr.write("%7.3f " %
                           getattr(conf, self.energy_used + "_energy" ))
            # end the line
            file_ptr.write("\n")
        if file_ptr != sys.stdout:
            file_ptr.close()


    def getInfoStr(self, comment='USER  AD>', ind=-1, rms=-1, ncl_to_write=1, include_max=False, report_all=False, include_dlgfilename_run=False):
        """Write a set of clusterings to a string instead of file 

A string summarizing the clusterings is returned with the format as described in
write method above:

Default:
binding\n 0   0   0   0  0.000  0.000  0.000 -14.645\n 1   0   1   0  2.449  0.000  0.000 -14.636\n 0   1   0   1  1.281  1.281  1.281 -14.424\n 1   1   1   1  2.548  1.014  1.014 -14.210\n

If comment is '#':
#binding\n#  0   0   0   0  0.000  0.000  0.000 -14.645\n#  1   0   1   0  2.449  0.000  0.000 -14.636\n#  0   1   0   1  1.281  1.281  1.281 -14.424\n#  1   1   1   1  2.548  1.014  1.014 -14.210\n

        """
        first = comment
        #if comment: first=comment
        clu_str = "%s"%first
        ind = int(ind)
        #corresponding filenames+ run numbers
        dlg_run_str = ""

        # make sure the keys happen in the same order
        sorted_keys = self.clustering_dict.keys()[:]
        sorted_keys.sort()
        if rms==-1:
            rms = sorted_keys[0]
            print "reporting rms %f clustering:"%rms
        if rms not in sorted_keys:
            print "no clustering exists at rms %f"%rms
            return "ERROR"
        # add tolerances and energy used on 1 line
        if report_all and len(sorted_keys)>1:
            for key in sorted_keys[1:]:
                clu_str += "%6s  " % str(key)
        # note which energy was used to cluster
        clu_str += "%6s\n" %str(self.energy_used)
        # add one line per cluster
        out_cl_num = len(self.clustering_dict[rms])
        n_omitted_cl = 0
        n_omitted_confs =0
        e_range_omitted_cl = 0
        nconf_omitted_cl = 0
        if ncl_to_write<out_cl_num:
            out_cl_num = ncl_to_write
            #record information about clusters not in infoStr
            #number of omitted clusters
            len_dict = len(self.clustering_dict[rms])
            n_omitted_cl = len_dict - ncl_to_write # output
            #print "set n_omitted_cl to ", n_omitted_cl
            #number of conformations
            #for cl in self.clustering_dict[rms][ncl_to_write:]:
            #print "len_dict-ncl_to_write=", len_dict - ncl_to_write
            for j in range(ncl_to_write, len_dict):
                cl = self.clustering_dict[rms][j]
                n_omitted_confs += len(cl)                # output
                #print "adding ", len(cl), " now n_omitted_confs=", n_omitted_confs
            #first and last omitted cluster energy
            first_omitted_e = self.clustering_dict[rms][ncl_to_write][0].binding_energy
            #??should last energy be best of last cluster or worst of last cluster??
            last_omitted_e = self.clustering_dict[rms][-1][-1].binding_energy
        for i in range(out_cl_num):
            #conf is lowest energy conformation in 'i-th' cluster
            conf = self.clustering_dict[rms][i][0]
            # append the rmsd from the overall reference conf (or seed)
            clu_str+="%s%.3f," %(first, self.get_distance(
                self.clustering_dict[rms][0][0],
                conf))
            # and finally the appropriate energy
            if self.energy_used=='energy':
                clu_str+="%.3f" % getattr(conf, "binding_energy" )
            else:
                clu_str+="%.3f" % getattr(conf, self.energy_used + "_energy" )
            # check whether ind corresponds to this conf
            #if ind==int(self.argsort[i]):
            if include_dlgfilename_run:
                dlg_run_str += "%s,%d" %(conf.filename, conf.run)
            #if self.data[ind]==conf:
            #    #clu_str += " * %d %d"%(ind, i)
            #    dlg_run_str += "1"
            #    #clu_str += "1,"
            #else:
            #    dlg_run_str += "0"
            ## end line for this conformation
            clu_str+="\n"
            dlg_run_str += "\n"
        if n_omitted_cl:
            clu_str="USER omitted %d clusters [%d confs]: be range %6.4f - %6.4f\n"%(n_omitted_cl,n_omitted_confs, first_omitted_e, last_omitted_e) + clu_str
        return clu_str, dlg_run_str



    def read(self, filename):
        """
1.5 2.0 binding
  0   0   0   0  0.000  0.000  0.000 -14.645
  1   0   1   0  2.449  0.000  0.000 -14.636
  0   1   0   1  1.281  1.281  1.281 -14.424
  1   1   1   1  2.548  1.014  1.014 -14.210
                   |      |      |
                   |      |      |-rmsd from cluster seed @ 2.0 tolerance
                   |      |-rmsd from cluster seed @ 1.0 tolerance
                   |-rmsd from overall reference structure      
        """
        file_ptr = open(filename)
        lines = file_ptr.readlines() # read the file
        file_ptr.close()

        d = self.clustering_dict # local copy

        # see if there's the last char is a 'd' or 'b'  or 'e'to denote energy
        word_list = string.split(lines[0])
        if word_list[-1][0] in ['b', 'd', 'e']: 
            file_energy_used = word_list[-1]
            ind = string.find(lines[0], file_energy_used)
            lines[0] = lines[0][:ind] # strip energy symbol
        else:
            file_energy_used = 'binding'

        # ??? make sure we're consistent with the argsort
        #check that self has done some clustering before this
        if len(self.clustering_dict)>0:
            assert file_energy_used[0] == self.energy_used[0], 'Cluster energy mismatch'
        else:
            #here the dlg had no clustering in it
            self.energy_used = file_energy_used
            #redo argsort with file_energy_used
            if file_energy_used[0] == 'd':
                energy_list = [conf.docking_energy for conf in self.data]
            else:
                energy_list = [conf.binding_energy for conf in self.data]
            # sort the conformations by energy
            self.argsort = Numeric.argsort(energy_list)
            #print "self.argsort=", self.argsort

        t_list = map(float, string.split(lines[0]))
        #t_list for the example is [0.5, 2.0]
        num_t = len(t_list)   #number of clusterings

        for tolerance in t_list: # initialize the keys
            if d.has_key(tolerance):
                raise RuntimeError, "overwriting existing clustering"
            c = d[tolerance] = Clustering()
            c.tolerance = tolerance

        # cx is the index into self.data, the list of conformatons
        # NEW FORMAT: 
        # 0   0   0   0  0.000  0.000  0.000 -14.645
        #first line has list of tolerances
        for cx, l in enumerate(lines[1:]):
            ll = l.split()
            #eg: 2 tolerances gives [ 0,  0,  0,  0, 0.000, 0.000, 0.000,-14.645]
            #num_t *2
            c_list = map(int, ll[:num_t*2])
            data_list = map(float, ll[num_t*2:])

            for t, i in zip(t_list, xrange(len(c_list)/2)):
                cluster_index = c_list[2*i]
                cluster_rank = c_list[2*i+1]
                conf = self.data[int(self.argsort[cx])]
                if cluster_rank == 0:
                    assert len(d[t]) == cluster_index
                    d[t].append(Cluster(conf))
                else:
                    # add conformation to its cluster
                    assert len(d[t][cluster_index]) == cluster_rank
                    d[t][cluster_index].append(conf)
                # tell the conformation what cluster(s) it belongs to...
                conf.cluster_dict[t] = (cluster_index, cluster_rank)
                conf.refRMS = data_list[0]
                conf.clRMS = data_list[1]
                #FIX THIS: this ends up in setting it for the last clustering value only
                #should this be set here????... i think not
                #setattr(conf, self.energy_used, data_list[-1])


    def rebuild_clusters(self, clusterLists, tolerance):
        """
        clusterLists: 
            ordered list of cl_lists built by parsing dlg
            first cl_list is for lowest energy cluster
            second cl_list is for next lowest energy cluster, etc
        cl_list:
            ordered lists of info for cluster's members:
                [clrank, rank, run, energy, clRMS, refRMS]
            within each cl_list, conformations are also ranked by energies.
        """

        clusters = Clustering()
        
        for cl_list in clusterLists:
            #new Cluster is initialized with seed=conf of lowest energy
            #initialize Cluster with 1st conf which is cl_list[0]
            # then 2 is index of run info which is 1 based
            ind = cl_list[0][2]-1
            conf = self.data[ind]
            c = Cluster(conf)
            init = 1
            for l in cl_list:
                #use l[0], run number, to get index to conf
                conf = self.data[l[2]-1]
                #print 'using conf ', self.data[l[2]-1]
                if not init:
                    c.append(conf)
                init = 0
                conf.energy, conf.clRMS, conf.refRMS = l[3:]
            clusters.append(c)
        #should this be extend or add or something?
        self.clustering_dict[tolerance] = clusters
        #tell all the conformations about this_clustering: 'clusters'
        for (cx, cl) in map(None, xrange(len(clusters)),
                                 clusters):
            for (rank, conf) in map(None, xrange(len(cl)), cl):
                conf.cluster_dict[tolerance] = (cx, rank)






#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/compute_consensus_maps_from_dlgs.py,v 1.9 2012/04/05 21:58:55 rhuey Exp $
#
# $Id: compute_consensus_maps_from_dlgs.py,v 1.9 2012/04/05 21:58:55 rhuey Exp $
#
import os, glob, numpy.oldnumeric as Numeric, math
from AutoDockTools.Docking import Docking
from PyAutoDock.AutoGrid import GridMap
from MolKit.molecule import Atom, AtomSet, Bond, Molecule
from MolKit.protein import Protein,Chain,ChainSet, Residue,ResidueSet
from MolKit.pdbWriter import PdbWriter

RT = 0.831*298.
from math import log, e
debug = False

def buildPdb(map_dict, npts, name='DlgBuilt', ctr=0, outputfile='results.pdb', 
                    scale=1.0):
    if debug: print "in buildPdb: tolerance=", tolerance
    name = 'DlgBuilt'
    mol = Protein(name=name)
    mol.curChain = Chain()
    mol.chains = ChainSet([mol.curChain])
    mol.curRes = Residue()
    mol.curChain.adopt(mol.curRes)
    mol.allAtoms = AtomSet()
    mol.curRes.atoms = mol.allAtoms
    nzpts=nypts=nxpts = npts
    #nxpts, nypts, nzpts = npts
    ctr = 0
    for ADtype, m in map_dict.items():
        if debug: 
            print "PROCESSING ", ADtype, " array:", max(m.ravel()), ':', min(m.ravel())
        vals = []
        tctr = 0  #for number of each type
        for z in range(nzpts):
            for y in range(nypts):
                for x in range(nxpts):
                    val = scale * abs(m[x,y,z])
                    vals.append(val)
                    #if abs(val)>.005:
                    if val>tolerance*scale:
                        ctr += 1
                        name = ADtype + str(ctr)
                        #version3:
                        #info_lo = (xcen - numxcells*spacing,
                        #    ycen - numycells*spacing, 
                        #    zcen - numzcells *spacing)
                        #using lower back pt of cube, i think
                        #xcoord = (x-info_lo[0])/spacing
                        #ycoord = (y-info_lo[1])/spacing
                        #zcoord = (z-info_lo[2])/spacing
                        #version2:
                        xcoord = (x-numxcells)*spacing + xcen
                        ycoord = (y-numycells)*spacing + ycen
                        zcoord = (z-numzcells)*spacing + zcen
                        coords = (xcoord,ycoord,zcoord)
                        tctr += 1
                    #    #print "addAtom: name=",name,"ADtype=", ADtype," val=", val, "coords=", coords,"ctr=", ctr
                        addAtom(mol, name, ADtype, val, coords, ctr)
        print "added ",tctr, '<-', ADtype, " atoms"
        if debug:
            print ADtype, ':', tctr , ' ', ctr
    print "total atoms=", ctr
    writer = PdbWriter()
    writer.write(outputfile, mol.allAtoms, records=['ATOM'])


def addAtom(mol, name, ADtype, value, coords, ctr):
    #if debug: print "in addAtom", value,
    res = mol.chains.residues[0]
    chemicalElement = ADtype[0] #???
    childIndex = ctr - 1
    top = mol
    newAt = Atom(name=name, parent=res, top=mol, 
                    chemicalElement=chemicalElement, 
                    childIndex=childIndex)
    newAt.temperatureFactor = value
    newAt.occupancy = value
    newAt.number = ctr
    newAt.conformation = 0
    newAt._coords = [list(coords)]
    newAt.hetatm = 0
    #if debug: print "added ", name, ctr, ':', newAt.full_name(),'-', newAt.parent.children.index(newAt)
    #update allAtoms attribute of this molecule
    mol.allAtoms = mol.chains.residues.atoms


def write_grid_map(filename, spacing, npts, center, score_array,maxval=10000 ):
    if debug:
        print "in write_grid_map ", filename
    stem = os.path.basename(filename).split('.')[0]
    # open and write the file
    fptr = open(filename, 'w')
    # line 1:
    ostr = "GRID_PARAMETER_FILE " + stem + ".gpf\n"
    fptr.write(ostr)
    # line 2:
    ostr = "GRID_DATA_FILE " + stem + ".maps.fld\n"
    fptr.write(ostr)
    # line 3:
    ostr = "MACROMOLECULE " + stem + ".pdbqt\n"
    fptr.write(ostr)
    # line 4:
    ostr = "SPACING " + str(spacing) + "\n"
    fptr.write(ostr)
    # line 5:
    ostr = "NELEMENTS %d %d %d\n" % (npts[0]-1, npts[1]-1, npts[2]-1)
    fptr.write(ostr)
    # line 6:
    ostr = "CENTER %f %f %f\n" % tuple(center)
    fptr.write(ostr)
    # now write the values:
    for z in range(npts[2]):
        for y in range(npts[1]):
            for x in range(npts[0]):
                value = score_array[x,y,z]
                ostr = "%.3f\n" % (value)
                fptr.write(ostr)
    # all done...
    fptr.close()


if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: compute_consensus_maps_from_dlgs.py "
        print "    Invoke this script in the directory containing dlg files"
        print "      which are results of multiple dockings to related targets"
        print
        print "    Description of command:"
        print "      Places atoms to build a molecule based on consensus maps "
        print "      which it builds from the results of multiple dockings to related targets"
        print "      By default creates 'results.pdb' based on lowest energy conformations"
        print "                                                 "
        print "    Optional parameters:"
        print "        [-n]    number of pts in each dimension (default is 101)"
        print "        [-s]    spacing between pts (default is 1.0 )"
        print "        [-t]    tolerance (default is .005 )"
        print "        [-o]    output pdb filename"
        print "                (default is 'results.pdb')"
        print "        [-a]    use all conformations  "
        print "                (default is to use only the one with the lowest energy)"
        print "        [-v]    verbose output"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'n:s:t:o:avh')
        if opt_list == []:
            usage()
            sys.exit(2)
    except getopt.GetoptError, msg:
        print 'compute_consensus_maps_from_dlgs.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    # optional parameters
    #-n: number of pts in each dimension 
    num_pts =  101
    #-s: spacing between pts 
    spacing =  1.0
    #-t: energy cutoff for including pt in calculation 
    tolerance =  0.005
    #-o outputfilename
    #-o outputfilename
    outputfilename = "results.pdb"
    #-a  use_all_conformations
    use_all_conformations = True
    #-verbose: chatty output
    verbose = None

    #'n:s:o:avh
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-n', '--n'):
            num_pts = a
            if verbose: print 'set number of pts in each dimension to ', a
        if o in ('-s', '--s'):
            spacing = float(a)
            if verbose: print 'set spacing to ', a
        if o in ('-o', '--o'):
            outputfilename = a
            if verbose: print 'set outputfilename to ', a
        if o in ('-a', '--a'):
            use_all_conformations = True
            if verbose: print 'set use_all_conformations to True'
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()

    #read all the docking logs in current directory, one by one
    dlg_list = glob.glob('./*.dlg')
    dockings = []
    #build a list of all atom types in all dlgs
    #it is assumed that all the dockings used the same grids
    ctr = 0
    at_types = {}
    for dlg in dlg_list:
        d = Docking()
        d.readDlg(dlg)
        ctr+= 1
        print ctr, ": read ", dlg
        dockings.append(d)
        for a in d.ligMol.allAtoms:
            at_types[a.autodock_element] = 0
    if debug: print 'at_types=', at_types.keys()
    d = dockings[0]  #get grid info from the first docking
    xcen, ycen, zcen = d.dlo_list[0].parser.center_pt
    #for the output maps...
    # nxgrid=nygrid=nzgrid=npts  ??is this required??
    nxpts = nypts = nzpts = int(num_pts)/2 * 2 + 1 #ensure an odd integer
    npts = (nxpts, nypts, nzpts)
    macroStem = d.dlo_list[0].macroStem
    # build list of all atom types
    all_types  = at_types.keys()
    # setup a map for each atom type
    maps = {}
    norm_maps = {}
    for t in all_types:
        maps[t] = Numeric.zeros((nxpts, nypts, nzpts)).astype('f')
        norm_maps[t] = Numeric.zeros((nxpts, nypts, nzpts)).astype('f')

    totalEnergy = 0
    #compute the totalEnergy
    if debug: print "computing the totalEnergy"
    N = 0
    for d in dockings:
        len_clusts = len(d.clusterer.clustering_dict.keys())
        if len_clusts:
            if use_all_conformations is True:
                confs = d.ch.conformations
                N += len(confs)
            else:
                key = d.clusterer.clustering_dict.keys()[0]
                confs = [d.clusterer.clustering_dict[key][0][0]]
                N += 1
        else:
            confs = [d.ch.conformations[0]]
        for c in confs:
            d.ch.set_conformation(c)
            deltaG = c.binding_energy
            c.ddG = e**(-deltaG/RT)
            totalEnergy += c.ddG
    #compute the probability of the individual conf._pi = c.ddG/totalEnergy
    print "computing the individual conf._pi"
    for d in dockings:
        len_clusts = len(d.clusterer.clustering_dict.keys())
        #if there is a clustering
        if len_clusts:
            if use_all_conformations is True:
                confs = d.ch.conformations
            else:
                key = d.clusterer.clustering_dict.keys()[0]
                confs = [d.clusterer.clustering_dict[key][0][0]]
        else:
            #if there was no clustering
            if use_all_conformations is True:
                confs = [d.ch.conformations]
            else:
                confs = [d.ch.conformations[0]]
        for c in confs:
            c._pi = c.ddG/totalEnergy
    #compute the probability of finding an atom at a specific pt in the maps
    print "totalEnergy=", totalEnergy
    print "computing the individual atom probabilities"
    for d in dockings:
        len_clusts = len(d.clusterer.clustering_dict.keys())
        if len_clusts:
            if use_all_conformations is True:
                confs = d.ch.conformations
            else:
                key = d.clusterer.clustering_dict.keys()[0]
                confs = [d.clusterer.clustering_dict[key][0][0]]
        else:
            if use_all_conformations is True:
                confs = [d.ch.conformations]
            else:
                confs = [d.ch.conformations[0]]
        for c in confs:
            #update the coordinates
            d.ch.set_conformation(c)
            for atom in d.ligMol.allAtoms:
                #get proper atomic grid
                m = maps[atom.autodock_element]
                #go to nearest pt in grid
                #FIX #1
                x,y,z = atom.coords
                numxcells = nxpts/2  #integer division
                numycells = nypts/2  #integer division
                numzcells = nzpts/2  #integer division
                #6/22: version3
                info_lo = (xcen - numxcells*spacing,
                            ycen - numycells*spacing, 
                            zcen - numzcells *spacing)
                #using lower back pt of cube, i think
                thispt = (int((x-info_lo[0])/spacing),
                          int((y-info_lo[1])/spacing),
                          int((z-info_lo[2])/spacing))
                #4/8:version 2
                #thispt = [  int(x/spacing-xcen+ spacing/2.0 +numxcells), 
                #            int(y/spacing-ycen+ spacing/2.0 +numycells), 
                #            int(z/spacing-zcen+ spacing/2.0 +numzcells)]
                #add this conf's probability to thispt ie: c._pi
                m[thispt] = m[thispt] + c._pi
    #calculate energies based on a Boltzman distribution (~normalizing the maps)
    maxval = -RT * log(tolerance)  #based on tolerance 0.005 cutoff
    if debug: print "tolerance ", tolerance, " yields maxval=", maxval
    minval = 10000
    RTtolerance = -RT*log(tolerance)
    for t in all_types:
        if debug: print "computing ", t, " normalized map"
        for z in range(nzpts):
            for y in range(nypts):
                for x in range(nxpts):
                    value = maps[t][x,y,z]
                    if value < tolerance: #-RTlog(0.005)=1312.06
                        Er = RTtolerance
                        #Er = 0.
                        #.00001
                        #Er = 2851.
                    else:
                        Er = min(-RT*log(value), 10000)
                        if Er<minval:
                            minval = Er
                    norm_maps[t][x,y,z] = Er
    #hd map: max=1290.887.0647.. min=0.0
    #maxval overal=1312.0647... minval=565.784
    # now adjust the values to the typical range of autogrid maps: -2<->2000:
    #maxminval = max(Numeric.array(minvals)) #560
    #minminval = min(Numeric.array(minvals)) #483
    rng = int(maxval-minval)  #-483
    #rng = int(half_maxval-minminval)  #-483
    if debug: print "maxval=", maxval, '- minval=', minval, ' so rng=', rng
    #print "half_maxval=", half_maxval, '- minminmval=', minminval, ' so rng=', rng
    #TRY1: rng = int(half_maxval-minminval)
    MAPS = {}
    zero_types = []
    for t in all_types:
        #do not process zero-maps
        nmap= norm_maps[t][:]
        #if max(nmap.ravel())!=min(nmap.ravel()):
        if max(nmap.ravel())==min(nmap.ravel()):
            del(norm_maps[t])
            zero_types.append(t) 
            print t, ' is a map of all zeros'
        else:
            nmap = nmap - maxval
            for z in range(nzpts):
                for y in range(nypts):
                    for x in range(nxpts):
                        value = nmap[x,y,z]
                        #if value<10:
                        nmap[x,y,z] = value/rng
                        #nmap[x,y,z] = 2.*value/rng
            MAPS[t] = nmap
    #write out the maps
    center = d.dlo_list[0].parser.center_pt
    if debug: print "writing the grid maps"
    #for atom_type, score_array in norm_maps.items():
    for atom_type, score_array in MAPS.items():
        filename = "%s.%s_combined.map" %(macroStem, atom_type)
        write_grid_map(filename, spacing, npts, center, score_array )
    #if debug: print "building the molecule"
    ####buildPdb(norm_maps, nxpts, name='DlgBuilt', ctr=0, outputfile='results.pdb')
    buildPdb(MAPS, nxpts, name='DlgBuilt', ctr=0, outputfile='results.pdb')

# To execute this command type:
# compute_consensus_maps_from_dlgs.py 



#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/write_lowest_energy_ligand.py,v 1.3 2012/01/31 18:00:39 rhuey Exp $
#
import os, glob

from MolKit import Read
from AutoDockTools.Docking import Docking
from mglutil.math.rmsd import RMSDCalculator




if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: write_lowest_energy_ligand.py -f dlgfilename"
        print
        print "    Description of command..."
        print "         -f     dlgfilename"
        print "    Optional parameters:"
        print "        [-m]    multiple dlg files"
        print "        [-N]    include information about best conformation"
        print "        [-v]    verbose output"
        print "        [-o pdbqt_filename] (output filename)"

    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'f:mNvo:h')
    except getopt.GetoptError, msg:
        print 'write_lowest_energy_ligand.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-f: dlgfilename
    dlgfilename =  None
    # optional parameters
    verbose = None
    #-o outputfilename
    outputfilename = None
    #-m multiple
    multiple = False
    #-N include_best_docking_info
    include_best_docking_info = False

    #'f:vmo:h'
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-f', '--f'):
            dlgfilename = a
            if verbose: print 'set dlgfilename to ', a
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-o', '--o'):
            outputfilename = a
            if verbose: print 'set outputfilename to ', a
        if o in ('-m', '--m'):
            multiple = True
            if verbose: print 'read all dlgs: set multiple to ', multiple
        if o in ('-N', '--N'):
            include_best_docking_info = True
            if verbose: print 'read all dlgs: set include_best_docking_info to ', include_best_docking_info
        if o in ('-h', '--'):
            usage()
            sys.exit()


    if not  dlgfilename:
        print 'write_lowest_energy_ligand: dlgfilename must be specified.'
        usage()
        sys.exit()

    #what about nucleic acids???

    rms_tolerance = 5.0
    d = Docking()
    if not multiple:
        d.readDlg(dlgfilename)
    else:    
        fl = glob.glob("*.dlg")
        for n in fl:
            d.readDlg(n)
    if verbose: print 'read ', dlgfilename
    if multiple:
        coords = d.ligMol.allAtoms.coords[:]
        d.clusterer.rmsTool = RMSDCalculator(coords)
        d.clusterer.make_clustering(rms_tolerance)
    key0 = d.clusterer.clustering_dict.keys()[0]
    conf0 = d.clusterer.clustering_dict[key0][0][0]
    #figure out its dlg filename
    for dlo in d.dlo_list:
        if conf0 in dlo.conformations:
            conf0_filename = dlo.parser.filename
    d.ch.set_conformation(conf0)
    #binding_energy, energy, intermol_energy: -427.09
    #electrostatic_energy +3.19
    #run 5
    #vdw_hb_desolv_energy -430.28
    #translation:(-30.497708000000003, -30.385196, -39.081098)
    #quaternion0: (-0.001463, -0.010084, -0.171606, -0.985113)
    #qtn_nx,qtn_ny, qtn_nz:(-0.008511,-0.058662,-0.998242)
    #qtn_ang_deg -19.797555
    #conf0.trn_x,  conf0.trn_y, conf0.trn_z: (-3.077708, 3.784804, -21.501098)
    #conf0.translation: (-30.497708000000003, -30.385196, -39.081098)
    parser = d.ligMol.parser
    lines = []
    #have to add newline character to lines read from dlg
    for l in parser.allLines:
        l+= '\n'
        lines.append(l)
    parser.allLines = lines
    coords = d.ligMol.allAtoms.coords
    if outputfilename is None:
        outputfilename = d.ligMol.name  + '_BE.pdbqt'
    parser.write_with_new_coords(coords, outputfilename) 
    if verbose:
        print 'wrote %s' %outputfilename
    if include_best_docking_info:
        fptr = open(outputfilename)
        lines = fptr.readlines()
        fptr.close()
        optr = open(outputfilename, 'w')
        ostr = "REMARK best energy run %d in file:%s\n" %(conf0.run, conf0_filename)
        optr.write(ostr)
        ostr = "REMARK binding_energy %10.5f\n" %(conf0.binding_energy)
        optr.write(ostr)
        ostr = "REMARK translation %10.4f %10.4f %10.4f\n" %(conf0.trn_x, conf0.trn_y, conf0.trn_z)
        optr.write(ostr)
        ostr = "REMARK quaternion0 %10.4f %10.4f %10.4f %10.4f\n" %(conf0.quaternion0)
        optr.write(ostr)
        for l in lines:
            optr.write(l)
        optr.close()
    print "wrote ", outputfilename
# To execute this command type:
# write_lowest_energy_ligand.py -f dlgfilename [-o outputfilename] -v







#########################################################################
#
# Date: Nov 2001 Authors: Michel Sanner
#
#    sanner@scripps.edu
#
#       The Scripps Research Institute (TSRI)
#       Molecular Graphics Lab
#       La Jolla, CA 92037, USA
#
# Copyright: Michel Sanner and TSRI
#
#########################################################################

from NetworkEditor.items import NetworkNode

from AutoDockTools.VisionInterface.Adt.autogrid_results import autogrid_results

import os

class AutogridResDirectoryBrowser(NetworkNode):
    """
    Browse local directory that contains Autogrid results.
    The directory will be compressed into a zip file.

    Input: Directory that contains autogrid results
    
    Output:
    port 1: autogrid_result object that contains info about the autogrid result directory
    port 2: path to compressed autogrid directory.  
    """
    
    def __init__(self, name='AutogridResDirectoryBrowser', **kw):
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )


        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )
        ip = self.inputPortsDescr
        ip.append(datatype='string', name='directory')

        self.widgetDescr['directory'] = {
            'class':'NEEntryWithDirectoryBrowser', 'master':'node', 'width':16,
            'initialValue':'', 'labelCfg':{'text':'Autogrid result directory: '}
            }

        op = self.outputPortsDescr
        op.append(datatype='autogrid_results', name='autogrid_result')
        op = self.outputPortsDescr
        op.append(datatype='string', name='zip_file_path')

        code = """def doit(self, directory):
    autogrid_result = autogrid_results(directory, "local")
          
    self.outputData(autogrid_result=autogrid_result, zip_file_path=autogrid_result.path)
"""
        self.setFunction(code)



#
#
#
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Tests/test_InteractionDetector.py,v 1.3 2010/06/14 23:49:18 rhuey Exp $
#
# $Id: test_InteractionDetector.py,v 1.3 2010/06/14 23:49:18 rhuey Exp $
#
#
#
# 
#

import unittest, os, sys
from AutoDockTools.InteractionDetector import InteractionDetector


class InteractionDetectorBaseTests(unittest.TestCase):
    def setUp(self):
        """
        instantiate an InteractionDetector
        """
        intD = InteractionDetector("hsg1V.pdbqt")
        self.assertEquals(intD.__class__, InteractionDetector)

        
class InteractionDetectorTests(InteractionDetectorBaseTests):

    def test_InteractionDetector(self):
        """
        check for number of hb atoms... etc 
        NOTE the atoms are found in random order so here we test for presence
        of various named atoms in the entire set of results...
        """
        self.intD = InteractionDetector("hsg1V.pdbqt")
        resStr = self.intD.processLigand("clean.pdbqt", outputfilename='filtered_clean.pdbqt')
        resStr_list = resStr.split('\n')
        all_strs = [ 'USER  AD> lig_hb_atoms : 5', 'USER  AD> clean: : INI 20:N4,H3~hsg1V:B:GLY27:O' , 'USER  AD> clean: : INI 20:O4,H3~hsg1V:B:ASP29:OD2' , 'USER  AD> hsg1V:B:ASP29:N,HN~clean: : INI 20:O4', 'USER  AD> clean: : INI 20:N4,H3~hsg1V:B:GLY27:O' , 'USER  AD> hsg1V:B:ASP29:N,HN~clean: : INI 20:O4', 'USER  AD> clean: : INI 20:O2,H2~hsg1V:B:ASP25:OD2', 'USER  AD> hsg1V:B:ARG8:NH2,HH22~clean: : INI 20:N5', 'USER  AD> macro_close_ats: 25', 'USER  AD> lig_close_ats: 18', 'USER  AD> USER clean: : INI 20:C2,C2,C1,C3,O4,C9,C2,C2,N5,H2,C1,C8,H3,C2,C3,H3,O2,C1']
        all_strs.append("USER  AD> clean: : INI 20:O2,H2~hsg1V:B:ASP25:OD1")
        all_strs.append("USER  AD> hsg1V:B:ARG8:NH2,HH22~clean: : INI 20:N5")
        self.assertEqual(resStr_list[0] in all_strs, True)
        self.assertEqual(resStr_list[1] in all_strs, True)
        self.assertEqual(resStr_list[2] in all_strs, True)
        self.assertEqual(resStr_list[3] in all_strs, True)
        self.assertEqual(resStr_list[4] in all_strs, True)
        self.assertEqual(resStr_list[5] in all_strs, True)
        self.assertEqual(resStr_list[6] in all_strs, True)
        self.assertEqual(resStr_list[8] in all_strs, True)
        #\nclean: : INI 20:N4,H3~hsg1V:B:GLY27:O\nclean: : INI 20:O4,H3~hsg1V:B:ASP29:OD2\nhsg1V:B:ASP29:N,HN~clean: : INI 20:O4\nclean: : INI 20:O2,H2~hsg1V:B:ASP25:OD1\nhsg1V:B:ARG8:NH2,HH22~clean: : INI 20:N5\nmacro_close_ats: 25\nUSER hsg1V:B:ARG8:HH22;hsg1V:A:ARG8:HH22;hsg1V:B:VAL32:CG2;hsg1V:B:GLY27:O;hsg1V:A:ASP25:CG;hsg1V:B:ASP30:HN;hsg1V:A:VAL82:CG1;hsg1V:A:ARG8:NH2;hsg1V:B:ASP25:CG;hsg1V:B:ASP29:HN;hsg1V:B:ALA28:CB;hsg1V:A:ARG8:CZ;hsg1V:B:ASP30:OD2;hsg1V:B:VAL82:CG2;hsg1V:B:ASP25:OD1;hsg1V:B:ASP30:CB;hsg1V:B:ASP25:OD2;hsg1V:A:ASP25:OD1;hsg1V:B:ASP29:OD2;hsg1V:B:ASP30:O;hsg1V:A:ARG8:HH21;hsg1V:B:ASP29:CG;hsg1V:B:PRO81:CG;hsg1V:B:PRO81:CB;hsg1V:B:ILE84:CD1\nlig_close_ats: 18\nUSER clean: : INI 20:C2,C2,C1,C3,O4,C9,C2,C2,N5,H2,C1,C8,H3,C2,C3,H3,O2,C1\n")
        


    def test_InteractionDetector_with_detect_pi(self):
        """
        check  detections of pi interactions as well as for number of hb atoms... etc 
        NOTE the atoms are found in random order so here we test for presence
        of various named atoms in the entire set of results...
        """
        self.intD = InteractionDetector("hsg1V.pdbqt", detect_pi=True)
        resStr = self.intD.processLigand("clean.pdbqt", outputfilename='filtered_clean.pdbqt')
        #self.assertEqual(resStr, 'lig_close_atoms:18\nlig_hb_atoms:5\npi_cation:1\n')
        resStr_list = resStr.split('\n')
        all_strs = ["USER  AD> lig_hb_atoms : 5", 'USER  AD> clean: : INI 20:O4,H3~hsg1V:B:ASP29:OD2', ]
        all_strs.append('USER  AD> clean: : INI 20:O2,H2~hsg1V:B:ASP25:OD1')
        all_strs.append('USER  AD> hsg1V:B:ARG8:NH2,HH22~clean: : INI 20:N5')
        all_strs.append('USER  AD> hsg1V:B:ASP29:N,HN~clean: : INI 20:O4')
        all_strs.append('USER  AD> clean: : INI 20:N4,H3~hsg1V:B:GLY27:O')
        all_strs.append('USER  AD> macro_close_ats: 25')
        all_strs.append('USER  AD> lig_close_ats: 18')
        all_strs.append('USER  AD> clean: : INI 20:') #H3,C1,O2,C2,C2,C8,H3,C9,C2,C1,C3,N5,C2,C2,C1,H2,C3,O4')
        #all_strs.append('USER clean: : INI 20:C1,O2,C2,C8,H3,C9,C2,C1,H3,C3,C2,O4,C2,C1,C2,H2,N5,C3')
        #all_strs.append('USER clean: : INI 20:O4,C1,O2,C2,C2,C8,H3,C9,C1,H3,C2,C3,N5,C2,C2,C1,H2,C3')
        #all_strs.append('USER clean: : INI 20:C1,O2,C2,C2,C8,H3,C9,C1,H3,C3,C2,C2,O4,C2,N5,C1,H2,C3')
        #all_strs.append('USER hsg1V:B:ILE84:CD1;hsg1V:B:PRO81:CB;hsg1V:A:ASP25:CG;hsg1V:B:VAL32:CG2;hsg1V:A:ARG8:NH2;hsg1V:B:ASP30:HN;hsg1V:B:ASP29:CG;hsg1V:A:ARG8:CZ;hsg1V:B:ARG8:HH22;hsg1V:A:ARG8:HH22;hsg1V:B:ASP30:CB;hsg1V:B:VAL82:CG2;hsg1V:A:ASP25:OD1;hsg1V:A:VAL82:CG1;hsg1V:B:ASP25:CG;hsg1V:B:ASP29:HN;hsg1V:B:ALA28:CB;hsg1V:B:ASP30:O;hsg1V:B:ASP30:OD2;hsg1V:B:ASP25:OD1;hsg1V:A:ARG8:HH21;hsg1V:B:ASP25:OD2;hsg1V:B:GLY27:O;hsg1V:B:PRO81:CG;hsg1V:B:ASP29:OD2')
        self.assertEqual(resStr_list[0], "USER  AD> lig_hb_atoms : 5")
        self.assertEqual(resStr_list[1] in all_strs, True)
        self.assertEqual(resStr_list[2] in all_strs, True)
        self.assertEqual(resStr_list[3] in all_strs, True)
        self.assertEqual(resStr_list[4] in all_strs, True)
        self.assertEqual(resStr_list[5] in all_strs, True)
        self.assertEqual(resStr_list[6] in all_strs, True)
        self.assertEqual(resStr_list[8] in all_strs, True)
        print resStr_list[9][:21] 
        #self.assertEqual(resStr_list[9][:21] in all_strs, True)
        #self.assertEqual(resStr_list[7] in all_strs, True)
        #\nhsg1V:B:ARG8:NH2,HH22~clean: : INI 20:N5\nclean: : INI 20:O4,H3~hsg1V:B:ASP29:OD2\nhsg1V:B:ASP29:N,HN~clean: : INI 20:O4\nclean: : INI 20:N4,H3~hsg1V:B:GLY27:O\nclean: : INI 20:O2,H2~hsg1V:B:ASP25:OD1\nmacro_close_ats: 25\nUSER hsg1V:B:GLY27:O;hsg1V:B:ASP25:CG;hsg1V:A:ARG8:HH21;hsg1V:B:ASP25:OD1;hsg1V:B:ASP29:CG;hsg1V:B:ASP29:HN;hsg1V:B:PRO81:CB;hsg1V:A:ARG8:HH22;hsg1V:B:ASP25:OD2;hsg1V:B:ASP30:CB;hsg1V:B:PRO81:CG;hsg1V:B:ALA28:CB;hsg1V:A:ASP25:CG;hsg1V:A:ARG8:NH2;hsg1V:A:ASP25:OD1;hsg1V:B:ASP30:OD2;hsg1V:B:VAL82:CG2;hsg1V:B:ASP30:O;hsg1V:B:ARG8:HH22;hsg1V:B:VAL32:CG2;hsg1V:B:ILE84:CD1;hsg1V:B:ASP30:HN;hsg1V:B:ASP29:OD2;hsg1V:A:VAL82:CG1;hsg1V:A:ARG8:CZ\nlig_close_ats: 18\nUSER clean: : INI 20:C3,H3,C1,N5,C2,C1,H3,H2,C8,C2,O4,C2,C2,C1,C2,C3,O2,C9\n")


    def test_no_interactions_InteractionDetector3(self):
        """
        
        """
        self.intD = InteractionDetector("1dwd_rec.pdbqt", detect_pi=True)
        resStr = self.intD.processLigand("clean.pdbqt", outputfilename='no_interactions_clean.pdbqt')
        self.assertEqual(resStr, 'USER  AD> lig_hb_atoms : 0\nUSER  AD> macro_close_ats: 0\nUSER  AD> lig_close_ats: 0\n')



if __name__ == '__main__':
    #unittest.main()
    test_cases = [
        'InteractionDetectorTests',
            ]
    unittest.main( argv=([__name__ ,] + test_cases) )


    



#!/usr/bin/env python

import os
from string import find

from MolKit import Read
from MolKit.pdbWriter import PdbqWriter




if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: pdbqt_to_pdbq.py -s filename"
        print
        print "    Description of command..."
        print "        [-f]    pdbqt_filename"
        print "    Optional parameters:"
        print "        [-o]    alternative pdbq_filename"
        print "        [-v]    verbose output"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'f:o:v')

    except getopt.GetoptError, msg:
        print 'pdbqt_to_pdbq.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-s: pdbqt_filename_stem
    pdbqt_filename =  None

    # optional parameters
    verbose = None
    pdbq_filename =  None

    #'f:o:v'
    for o, a in opt_list:
        if o in ('-f', '--f'):
            pdbqt_filename = a
            stem = pdbqt_filename.split('.')[0]
            pdbq_filename = stem + ".pdbq"
            if verbose: print 'set pdbqt_filename to ', pdbqt_filename
        if o in ('-o', '--o'):
            pdbq_filename = a 
            if verbose: 
                print 'set output pdbq_filename to ', a
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()


    if not pdbqt_filename:
        print 'pdbqt_to_pdbq: pdbqt_filename must be specified.'
        usage()
        sys.exit()

    mols = Read(pdbqt_filename)
    if verbose: print 'read ', pdbqt_filename
    mol = mols[0]
    mol.buildBondsByDistance()
    #fix number for problem files with alternative positions
    mol.allAtoms.number = range(1, len(mol.allAtoms)+1)

    #pdb_filename = mol.name + '.pdb'
    writer = PdbqWriter()
    #writer.write(pdbq_filename, mol.allAtoms, records=['ATOM', 'HETATM'])
    fptr = open(pdbq_filename, 'w')
    ctr = 0
    for l in mol.parser.allLines:
        if l.find('ATOM')==0 or l.find('HETATM')==0:
            at = mol.allAtoms[ctr]
            if at.autodock_element=='A':
                if len(at.name)==1:
                    at.name='A'
                else:
                    at.name = 'A' + at.name[1:]
            writer.write_atom(fptr, mol.allAtoms[ctr])
            ctr = ctr + 1
        else:
            fptr.write(l)
    if verbose:
        print "wrote ", ctr, " atoms to", pdbq_filename
    

# To execute this command type:
# pdbqt_to_pdb.py -f pdbqt_filename_stem [-o outputfilename] -v







########################################################################
#
#    Vision Macro - Python source code - file generated by vision
#    Tuesday 29 June 2010 18:26:50 
#    
#       The Scripps Research Institute (TSRI)
#       Molecular Graphics Lab
#       La Jolla, CA 92037, USA
#
# Copyright: Daniel Stoffler, Michel Sanner and TSRI
#   
# revision: Guillaume Vareille
#  
#########################################################################
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/VisionInterface/Adt/Macro/VirtualScreening.py,v 1.12 2010/07/02 00:23:01 jren Exp $
#
# $Id: VirtualScreening.py,v 1.12 2010/07/02 00:23:01 jren Exp $
#

from NetworkEditor.macros import MacroNode
from NetworkEditor.macros import MacroNode
class VirtualScreening(MacroNode):

    '''
    Macro node that contains PrepareReceptor, ComputeGrids and AutodockVS.

    Inputs:
    port 1: receptor object
    port 2: LigandDB object
    port 3: gpf_template object
    port 4: dpf_template object
    port 5: boolean value, the user should use CheckButton from Standard library to pass the value 
            to this port. 
            If checked, then Prepare Receptor will preserve all input charges ie. do not add new charges 
            (default is addition of gasteiger charges)

    Outputs:
    port 1: URL to Autodock virtual screening results
    '''

    def __init__(self, constrkw={}, name='VirtualScreening', **kw):
        kw['name'] = name
        apply( MacroNode.__init__, (self,), kw)

    def beforeAddingToNetwork(self, net):
        MacroNode.beforeAddingToNetwork(self, net)
        from AutoDockTools.VisionInterface.Adt import Adt
        from WebServices.VisionInterface.WSNodes import wslib
        from Vision.StandardNodes import stdlib
        net.getEditor().addLibraryInstance(wslib,"WebServices.VisionInterface.WSNodes", "wslib")
        from WebServices.VisionInterface.WSNodes import addOpalServerAsCategory
        try:
            addOpalServerAsCategory("http://ws.nbcr.net/opal2", replace=False)
        except:
            pass
        try:
            addOpalServerAsCategory("http://kryptonite.nbcr.net/opal2", replace=False)
        except:
            pass

    def afterAddingToNetwork(self):
        masterNet = self.macroNetwork
        from NetworkEditor.macros import MacroNode
        MacroNode.afterAddingToNetwork(self)
        from AutoDockTools.VisionInterface.Adt import Adt
        from WebServices.VisionInterface.WSNodes import wslib
        from Vision.StandardNodes import stdlib
        ## building macro network ##
        VirtualScreening_0 = self
        from traceback import print_exc
        from AutoDockTools.VisionInterface.Adt import Adt
        from WebServices.VisionInterface.WSNodes import wslib
        from Vision.StandardNodes import stdlib
        masterNet.getEditor().addLibraryInstance(wslib,"WebServices.VisionInterface.WSNodes", "wslib")
        from WebServices.VisionInterface.WSNodes import addOpalServerAsCategory
        try:
            addOpalServerAsCategory("http://ws.nbcr.net/opal2", replace=False)
        except:
            pass
        try:
            addOpalServerAsCategory("http://kryptonite.nbcr.net/opal2", replace=False)
        except:
            pass
        try:
            ## saving node input Ports ##
            input_Ports_1 = self.macroNetwork.ipNode
            apply(input_Ports_1.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
        except:
            print "WARNING: failed to restore MacroInputNode named input Ports in network self.macroNetwork"
            print_exc()
            input_Ports_1=None

        try:
            ## saving node output Ports ##
            output_Ports_2 = self.macroNetwork.opNode
            apply(output_Ports_2.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
            output_Ports_2.move(217, 346)
        except:
            print "WARNING: failed to restore MacroOutputNode named output Ports in network self.macroNetwork"
            print_exc()
            output_Ports_2=None

        try:
            ## saving node PrepareReceptor ##
            from Adt.Macro.PrepareReceptor import PrepareReceptor
            PrepareReceptor_3 = PrepareReceptor(constrkw={}, name='PrepareReceptor', library=Adt)
            self.macroNetwork.addNode(PrepareReceptor_3,96,114)
            apply(PrepareReceptor_3.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
            Pdb2pqrWS_6 = PrepareReceptor_3.macroNetwork.nodes[2]
            Pdb2pqrOpalService_ws_nbcr_net_10 = Pdb2pqrWS_6.macroNetwork.nodes[3]
            Pdb2pqrOpalService_ws_nbcr_net_10.inputPortByName['noopt'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_10.inputPortByName['phi'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_10.inputPortByName['psi'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_10.inputPortByName['verbose'].widget.set(1, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_10.inputPortByName['chain'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_10.inputPortByName['nodebump'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_10.inputPortByName['chi'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_10.inputPortByName['ligand'].widget.set(r"", run=False)
            Pdb2pqrOpalService_ws_nbcr_net_10.inputPortByName['hbond'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_10.inputPortByName['with_ph'].widget.set(r"", run=False)
            apply(Pdb2pqrOpalService_ws_nbcr_net_10.inputPortByName['forcefield'].widget.configure, (), {'choices': ('AMBER', 'CHARMM', 'PARSE', 'TYL06')})
            Pdb2pqrOpalService_ws_nbcr_net_10.inputPortByName['forcefield'].widget.set(r"AMBER", run=False)
            Pdb2pqrOpalService_ws_nbcr_net_10.inputPortByName['clean'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_10.inputPortByName['inId'].widget.set(r"", run=False)
            Pdb2pqrOpalService_ws_nbcr_net_10.inputPortByName['apbs_input'].widget.set(0, run=False)
            apply(Pdb2pqrOpalService_ws_nbcr_net_10.inputPortByName['ffout'].widget.configure, (), {'choices': ('AMBER', 'CHARMM', 'PARSE', 'TYL06')})
            Pdb2pqrOpalService_ws_nbcr_net_10.inputPortByName['ffout'].widget.set(r"", run=False)
            Pdb2pqrOpalService_ws_nbcr_net_10.inputPortByName['localRun'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_10.inputPortByName['rama'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_10.inputPortByName['execPath'].widget.set(r"", run=False)
            Pdb2pqrOpalService_ws_nbcr_net_10.inputPortByName['assign_only'].widget.set(0, run=False)
            GetURLFromList_11 = Pdb2pqrWS_6.macroNetwork.nodes[4]
            GetURLFromList_11.inputPortByName['ext'].widget.set(r"pqr", run=False)

            ## saving connections for network Pdb2pqrWS ##
            Pdb2pqrWS_6.macroNetwork.freeze()
            Pdb2pqrWS_6.macroNetwork.unfreeze()

            ## modifying MacroInputNode dynamic ports
            input_Ports_7 = Pdb2pqrWS_6.macroNetwork.ipNode
            input_Ports_7.outputPorts[1].configure(name='CheckFileFormat_value')

            ## modifying MacroOutputNode dynamic ports
            output_Ports_8 = Pdb2pqrWS_6.macroNetwork.opNode
            output_Ports_8.inputPorts[1].configure(singleConnection='auto')
            output_Ports_8.inputPorts[2].configure(singleConnection='auto')
            output_Ports_8.inputPorts[1].configure(name='UpdateReceptor_receptor_obj')
            output_Ports_8.inputPorts[2].configure(name='UpdateReceptor_pdb2pqr_result')
            Pdb2pqrWS_6.inputPorts[0].configure(name='CheckFileFormat_value')
            Pdb2pqrWS_6.inputPorts[0].configure(datatype='receptor')
            ## configure MacroNode input ports
            Pdb2pqrWS_6.outputPorts[0].configure(name='UpdateReceptor_receptor_obj')
            Pdb2pqrWS_6.outputPorts[0].configure(datatype='receptor')
            Pdb2pqrWS_6.outputPorts[1].configure(name='UpdateReceptor_pdb2pqr_result')
            Pdb2pqrWS_6.outputPorts[1].configure(datatype='string')
            ## configure MacroNode output ports
            Pdb2pqrWS_6.shrink()
            PrepareReceptorWS_13 = PrepareReceptor_3.macroNetwork.nodes[3]
            PrepareReceptorOpalService_ws_nbcr_net_17 = PrepareReceptorWS_13.macroNetwork.nodes[3]
            PrepareReceptorOpalService_ws_nbcr_net_17.inputPortByName['o'].widget.set(r"", run=False)
            PrepareReceptorOpalService_ws_nbcr_net_17.inputPortByName['v'].widget.set(0, run=False)
            PrepareReceptorOpalService_ws_nbcr_net_17.inputPortByName['localRun'].widget.set(0, run=False)
            PrepareReceptorOpalService_ws_nbcr_net_17.inputPortByName['execPath'].widget.set(r"", run=False)
            GetURLFromList_18 = PrepareReceptorWS_13.macroNetwork.nodes[4]
            GetURLFromList_18.inputPortByName['ext'].widget.set(r"pdbqt", run=False)
            DownloadToFile_19 = PrepareReceptorWS_13.macroNetwork.nodes[5]
            DownloadToFile_19.inputPortByName['overwrite'].widget.set(1, run=False)

            ## saving connections for network PrepareReceptorWS ##
            PrepareReceptorWS_13.macroNetwork.freeze()
            PrepareReceptorWS_13.macroNetwork.unfreeze()

            ## modifying MacroInputNode dynamic ports
            input_Ports_14 = PrepareReceptorWS_13.macroNetwork.ipNode
            input_Ports_14.outputPorts[1].configure(name='CheckFileFormat_value')
            input_Ports_14.outputPorts[2].configure(name='PrepareReceptorOpalService_ws_nbcr_net_C')

            ## modifying MacroOutputNode dynamic ports
            output_Ports_15 = PrepareReceptorWS_13.macroNetwork.opNode
            output_Ports_15.inputPorts[1].configure(singleConnection='auto')
            output_Ports_15.inputPorts[2].configure(singleConnection='auto')
            output_Ports_15.inputPorts[1].configure(name='UpdateReceptor_receptor_prepared_obj')
            output_Ports_15.inputPorts[2].configure(name='UpdateReceptor_receptor_result')
            PrepareReceptorWS_13.inputPorts[0].configure(name='CheckFileFormat_value')
            PrepareReceptorWS_13.inputPorts[0].configure(datatype='receptor')
            PrepareReceptorWS_13.inputPorts[1].configure(name='PrepareReceptorOpalService_ws_nbcr_net_C')
            PrepareReceptorWS_13.inputPorts[1].configure(datatype='boolean')
            ## configure MacroNode input ports
            PrepareReceptorWS_13.outputPorts[0].configure(name='UpdateReceptor_receptor_prepared_obj')
            PrepareReceptorWS_13.outputPorts[0].configure(datatype='receptor_prepared')
            PrepareReceptorWS_13.outputPorts[1].configure(name='UpdateReceptor_receptor_result')
            PrepareReceptorWS_13.outputPorts[1].configure(datatype='string')
            ## configure MacroNode output ports
            PrepareReceptorWS_13.shrink()

            ## saving connections for network PrepareReceptor ##
            PrepareReceptor_3.macroNetwork.freeze()
            PrepareReceptor_3.macroNetwork.unfreeze()

            ## modifying MacroInputNode dynamic ports
            input_Ports_4 = PrepareReceptor_3.macroNetwork.ipNode
            input_Ports_4.outputPorts[1].configure(name='Pdb2pqrWS_CheckFileFormat_value')
            input_Ports_4.outputPorts[2].configure(name='PrepareReceptorWS_PrepareReceptorOpalService_ws_nbcr_net_C')

            ## modifying MacroOutputNode dynamic ports
            output_Ports_5 = PrepareReceptor_3.macroNetwork.opNode
            output_Ports_5.inputPorts[1].configure(singleConnection='auto')
            output_Ports_5.inputPorts[2].configure(singleConnection='auto')
            output_Ports_5.inputPorts[1].configure(name='PrepareReceptorWS_UpdateReceptor_receptor_prepared_obj')
            output_Ports_5.inputPorts[2].configure(name='PrepareReceptorWS_UpdateReceptor_receptor_result')
            PrepareReceptor_3.inputPorts[0].configure(name='Pdb2pqrWS_CheckFileFormat_value')
            PrepareReceptor_3.inputPorts[0].configure(datatype='receptor')
            PrepareReceptor_3.inputPorts[1].configure(name='PrepareReceptorWS_PrepareReceptorOpalService_ws_nbcr_net_C')
            PrepareReceptor_3.inputPorts[1].configure(datatype='boolean')
            ## configure MacroNode input ports
            PrepareReceptor_3.outputPorts[0].configure(name='PrepareReceptorWS_UpdateReceptor_receptor_prepared_obj')
            PrepareReceptor_3.outputPorts[0].configure(datatype='receptor_prepared')
            PrepareReceptor_3.outputPorts[1].configure(name='PrepareReceptorWS_UpdateReceptor_receptor_result')
            PrepareReceptor_3.outputPorts[1].configure(datatype='string')
            ## configure MacroNode output ports
            PrepareReceptor_3.shrink()
            apply(PrepareReceptor_3.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
        except:
            print "WARNING: failed to restore PrepareReceptor named PrepareReceptor in network self.macroNetwork"
            print_exc()
            PrepareReceptor_3=None

        try:
            ## saving node ComputeGrids ##
            from Adt.Macro.ComputeGrids import ComputeGrids
            ComputeGrids_21 = ComputeGrids(constrkw={}, name='ComputeGrids', library=Adt)
            self.macroNetwork.addNode(ComputeGrids_21,187,205)
            apply(ComputeGrids_21.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
            prepareGPF_kryptonite_nbcr_net_25 = ComputeGrids_21.macroNetwork.nodes[3]
            prepareGPF_kryptonite_nbcr_net_25.inputPortByName['singlelib'].widget.set(r"", run=False)
            prepareGPF_kryptonite_nbcr_net_25.inputPortByName['r_url'].widget.set(r"", run=False)
            prepareGPF_kryptonite_nbcr_net_25.inputPortByName['zpoints'].widget.set(r"", run=False)
            prepareGPF_kryptonite_nbcr_net_25.inputPortByName['filter_file_url'].widget.set(r"", run=False)
            apply(prepareGPF_kryptonite_nbcr_net_25.inputPortByName['lib'].widget.configure, (), {'choices': ('sample', 'NCIDS_SC', 'NCI_DS1', 'NCI_DS2', 'oldNCI', 'human_metabolome', 'chembridge_building_blocks', 'drugbank_nutraceutics', 'drugbank_smallmol', 'fda_approved')})
            prepareGPF_kryptonite_nbcr_net_25.inputPortByName['lib'].widget.set(r"", run=False)
            prepareGPF_kryptonite_nbcr_net_25.inputPortByName['ypoints'].widget.set(r"", run=False)
            prepareGPF_kryptonite_nbcr_net_25.inputPortByName['xcenter'].widget.set(r"auto", run=False)
            prepareGPF_kryptonite_nbcr_net_25.inputPortByName['p'].widget.set(r"", run=False)
            prepareGPF_kryptonite_nbcr_net_25.inputPortByName['o'].widget.set(r"", run=False)
            prepareGPF_kryptonite_nbcr_net_25.inputPortByName['zcenter'].widget.set(r"auto", run=False)
            prepareGPF_kryptonite_nbcr_net_25.inputPortByName['v'].widget.set(0, run=False)
            prepareGPF_kryptonite_nbcr_net_25.inputPortByName['userlib'].widget.set(r"", run=False)
            prepareGPF_kryptonite_nbcr_net_25.inputPortByName['xpoints'].widget.set(r"", run=False)
            prepareGPF_kryptonite_nbcr_net_25.inputPortByName['localRun'].widget.set(0, run=False)
            prepareGPF_kryptonite_nbcr_net_25.inputPortByName['ycenter'].widget.set(r"auto", run=False)
            prepareGPF_kryptonite_nbcr_net_25.inputPortByName['execPath'].widget.set(r"", run=False)
            autogrid_kryptonite_nbcr_net_26 = ComputeGrids_21.macroNetwork.nodes[4]
            autogrid_kryptonite_nbcr_net_26.inputPortByName['infile_url'].widget.set(r"", run=False)
            autogrid_kryptonite_nbcr_net_26.inputPortByName['l'].widget.set(r"output.glg", run=False)
            autogrid_kryptonite_nbcr_net_26.inputPortByName['o'].widget.set(0, run=False)
            autogrid_kryptonite_nbcr_net_26.inputPortByName['p'].widget.set(r"", run=False)
            autogrid_kryptonite_nbcr_net_26.inputPortByName['localRun'].widget.set(0, run=False)
            autogrid_kryptonite_nbcr_net_26.inputPortByName['execPath'].widget.set(r"", run=False)
            GetURLFromList_29 = ComputeGrids_21.macroNetwork.nodes[7]
            GetURLFromList_29.inputPortByName['ext'].widget.set(r"gpf", run=False)

            ## saving connections for network ComputeGrids ##
            ComputeGrids_21.macroNetwork.freeze()
            ComputeGrids_21.macroNetwork.unfreeze()

            ## modifying MacroInputNode dynamic ports
            input_Ports_22 = ComputeGrids_21.macroNetwork.ipNode
            input_Ports_22.outputPorts[1].configure(name='GetComputeGridsInputs_ligands')
            input_Ports_22.outputPorts[2].configure(name='GetComputeGridsInputs_receptor_pdbqt')
            input_Ports_22.outputPorts[3].configure(name='GetComputeGridsInputs_gpf_obj')

            ## modifying MacroOutputNode dynamic ports
            output_Ports_23 = ComputeGrids_21.macroNetwork.opNode
            output_Ports_23.inputPorts[1].configure(singleConnection='auto')
            output_Ports_23.inputPorts[2].configure(singleConnection='auto')
            output_Ports_23.inputPorts[1].configure(name='MakeAutogridResultObj_autogrid_result_obj')
            output_Ports_23.inputPorts[2].configure(name='GetMainURLFromList_newurl')
            ComputeGrids_21.inputPorts[0].configure(name='GetComputeGridsInputs_ligands')
            ComputeGrids_21.inputPorts[0].configure(datatype='LigandDB')
            ComputeGrids_21.inputPorts[1].configure(name='GetComputeGridsInputs_receptor_pdbqt')
            ComputeGrids_21.inputPorts[1].configure(datatype='receptor_prepared')
            ComputeGrids_21.inputPorts[2].configure(name='GetComputeGridsInputs_gpf_obj')
            ComputeGrids_21.inputPorts[2].configure(datatype='gpf_template')
            ## configure MacroNode input ports
            ComputeGrids_21.outputPorts[0].configure(name='MakeAutogridResultObj_autogrid_result_obj')
            ComputeGrids_21.outputPorts[0].configure(datatype='autogrid_results')
            ComputeGrids_21.outputPorts[1].configure(name='GetMainURLFromList_newurl')
            ComputeGrids_21.outputPorts[1].configure(datatype='string')
            ## configure MacroNode output ports
            ComputeGrids_21.shrink()
            apply(ComputeGrids_21.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
        except:
            print "WARNING: failed to restore ComputeGrids named ComputeGrids in network self.macroNetwork"
            print_exc()
            ComputeGrids_21=None

        try:
            ## saving node AutodockVS ##
            from Adt.Macro.AutodockVS import AutodockVS
            AutodockVS_30 = AutodockVS(constrkw={}, name='AutodockVS', library=Adt)
            self.macroNetwork.addNode(AutodockVS_30,234,289)
            apply(AutodockVS_30.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
            autodock_kryptonite_nbcr_net_34 = AutodockVS_30.macroNetwork.nodes[3]
            autodock_kryptonite_nbcr_net_34.inputPortByName['ga_run'].widget.set(r"", run=False)
            apply(autodock_kryptonite_nbcr_net_34.inputPortByName['lib'].widget.configure, (), {'choices': ('sample', 'NCIDS_SC', 'NCI_DS1', 'NCI_DS2', 'human_metabolome', 'chembridge_building_blocks', 'drugbank_nutraceutics', 'drugbank_smallmol', 'fda_approved')})
            autodock_kryptonite_nbcr_net_34.inputPortByName['lib'].widget.set(r"", run=False)
            autodock_kryptonite_nbcr_net_34.inputPortByName['filter_file_url'].widget.set(r"", run=False)
            autodock_kryptonite_nbcr_net_34.inputPortByName['ga_num_evals'].widget.set(r"", run=False)
            apply(autodock_kryptonite_nbcr_net_34.inputPortByName['sched'].widget.configure, (), {'choices': ('SGE', 'CSF')})
            autodock_kryptonite_nbcr_net_34.inputPortByName['sched'].widget.set(r"SGE", run=False)
            autodock_kryptonite_nbcr_net_34.inputPortByName['ga_num_generations'].widget.set(r"", run=False)
            autodock_kryptonite_nbcr_net_34.inputPortByName['userlib'].widget.set(r"", run=False)
            autodock_kryptonite_nbcr_net_34.inputPortByName['ga_pop_size'].widget.set(r"", run=False)
            autodock_kryptonite_nbcr_net_34.inputPortByName['localRun'].widget.set(0, run=False)
            autodock_kryptonite_nbcr_net_34.inputPortByName['email'].widget.set(r"", run=False)
            autodock_kryptonite_nbcr_net_34.inputPortByName['execPath'].widget.set(r"", run=False)

            ## saving connections for network AutodockVS ##
            AutodockVS_30.macroNetwork.freeze()
            AutodockVS_30.macroNetwork.unfreeze()

            ## modifying MacroInputNode dynamic ports
            input_Ports_31 = AutodockVS_30.macroNetwork.ipNode
            input_Ports_31.outputPorts[1].configure(name='PrepareADVSInputs_ligands')
            input_Ports_31.outputPorts[2].configure(name='PrepareADVSInputs_autogrid_results')
            input_Ports_31.outputPorts[3].configure(name='PrepareADVSInputs_dpf_template_obj')

            ## modifying MacroOutputNode dynamic ports
            output_Ports_32 = AutodockVS_30.macroNetwork.opNode
            output_Ports_32.inputPorts[1].configure(singleConnection='auto')
            output_Ports_32.inputPorts[1].configure(name='GetMainURLFromList_newurl')
            AutodockVS_30.inputPorts[0].configure(name='PrepareADVSInputs_ligands')
            AutodockVS_30.inputPorts[0].configure(datatype='LigandDB')
            AutodockVS_30.inputPorts[1].configure(name='PrepareADVSInputs_autogrid_results')
            AutodockVS_30.inputPorts[1].configure(datatype='autogrid_results')
            AutodockVS_30.inputPorts[2].configure(name='PrepareADVSInputs_dpf_template_obj')
            AutodockVS_30.inputPorts[2].configure(datatype='dpf_template')
            ## configure MacroNode input ports
            AutodockVS_30.outputPorts[0].configure(name='GetMainURLFromList_newurl')
            AutodockVS_30.outputPorts[0].configure(datatype='string')
            ## configure MacroNode output ports
            AutodockVS_30.shrink()
            apply(AutodockVS_30.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
        except:
            print "WARNING: failed to restore AutodockVS named AutodockVS in network self.macroNetwork"
            print_exc()
            AutodockVS_30=None

        #self.macroNetwork.run()
        self.macroNetwork.freeze()

        ## saving connections for network VirtualScreening ##
        input_Ports_1 = self.macroNetwork.ipNode
        if input_Ports_1 is not None and PrepareReceptor_3 is not None:
            try:
                self.macroNetwork.connectNodes(
                    input_Ports_1, PrepareReceptor_3, "new", "Pdb2pqrWS_CheckFileFormat_value", blocking=True
                    , splitratio=[0.73953989601548642, 0.35343455296743803])
            except:
                print "WARNING: failed to restore connection between input_Ports_1 and PrepareReceptor_3 in network self.macroNetwork"
        if input_Ports_1 is not None and ComputeGrids_21 is not None:
            try:
                self.macroNetwork.connectNodes(
                    input_Ports_1, ComputeGrids_21, "new", "GetComputeGridsInputs_ligands", blocking=True
                    , splitratio=[0.46529144721861959, 0.40210922169933605])
            except:
                print "WARNING: failed to restore connection between input_Ports_1 and ComputeGrids_21 in network self.macroNetwork"
        if input_Ports_1 is not None and AutodockVS_30 is not None:
            try:
                self.macroNetwork.connectNodes(
                    input_Ports_1, AutodockVS_30, "ComputeGrids_GetComputeGridsInputs_ligands", "PrepareADVSInputs_ligands", blocking=True
                    , splitratio=[0.48337865988456058, 0.41105508936996049])
            except:
                print "WARNING: failed to restore connection between input_Ports_1 and AutodockVS_30 in network self.macroNetwork"
        if PrepareReceptor_3 is not None and ComputeGrids_21 is not None:
            try:
                self.macroNetwork.connectNodes(
                    PrepareReceptor_3, ComputeGrids_21, "PrepareReceptorWS_UpdateReceptor_receptor_prepared_obj", "GetComputeGridsInputs_receptor_pdbqt", blocking=True
                    , splitratio=[0.32656364590133868, 0.43316260938324158])
            except:
                print "WARNING: failed to restore connection between PrepareReceptor_3 and ComputeGrids_21 in network self.macroNetwork"
        if ComputeGrids_21 is not None and AutodockVS_30 is not None:
            try:
                self.macroNetwork.connectNodes(
                    ComputeGrids_21, AutodockVS_30, "MakeAutogridResultObj_autogrid_result_obj", "PrepareADVSInputs_autogrid_results", blocking=True
                    , splitratio=[0.25195726140883218, 0.37435524480319915])
            except:
                print "WARNING: failed to restore connection between ComputeGrids_21 and AutodockVS_30 in network self.macroNetwork"
        if input_Ports_1 is not None and ComputeGrids_21 is not None:
            try:
                self.macroNetwork.connectNodes(
                    input_Ports_1, ComputeGrids_21, "new", "GetComputeGridsInputs_gpf_obj", blocking=True
                    , splitratio=[0.3610566319618409, 0.20507643166097597])
            except:
                print "WARNING: failed to restore connection between input_Ports_1 and ComputeGrids_21 in network self.macroNetwork"
        if input_Ports_1 is not None and AutodockVS_30 is not None:
            try:
                self.macroNetwork.connectNodes(
                    input_Ports_1, AutodockVS_30, "new", "PrepareADVSInputs_dpf_template_obj", blocking=True
                    , splitratio=[0.41221000935616448, 0.71284031297323347])
            except:
                print "WARNING: failed to restore connection between input_Ports_1 and AutodockVS_30 in network self.macroNetwork"
        output_Ports_2 = self.macroNetwork.opNode
        if AutodockVS_30 is not None and output_Ports_2 is not None:
            try:
                self.macroNetwork.connectNodes(
                    AutodockVS_30, output_Ports_2, "GetMainURLFromList_newurl", "new", blocking=True
                    , splitratio=[0.28632220295352073, 0.59933112675886213])
            except:
                print "WARNING: failed to restore connection between AutodockVS_30 and output_Ports_2 in network self.macroNetwork"
        if input_Ports_1 is not None and PrepareReceptor_3 is not None:
            try:
                self.macroNetwork.connectNodes(
                    input_Ports_1, PrepareReceptor_3, "new", "PrepareReceptorWS_PrepareReceptorOpalService_ws_nbcr_net_C", blocking=True
                    , splitratio=[0.66210822612399844, 0.69280049494960727])
            except:
                print "WARNING: failed to restore connection between input_Ports_1 and PrepareReceptor_3 in network self.macroNetwork"
        self.macroNetwork.runOnNewData.value = False

        ## modifying MacroInputNode dynamic ports
        input_Ports_1 = self.macroNetwork.ipNode
        input_Ports_1.outputPorts[1].configure(name='PrepareReceptor_Pdb2pqrWS_CheckFileFormat_value')
        input_Ports_1.outputPorts[2].configure(name='ComputeGrids_GetComputeGridsInputs_ligands')
        input_Ports_1.outputPorts[3].configure(name='ComputeGrids_GetComputeGridsInputs_gpf_obj')
        input_Ports_1.outputPorts[4].configure(name='AutodockVS_PrepareADVSInputs_dpf_template_obj')
        input_Ports_1.outputPorts[5].configure(name='PrepareReceptor_PrepareReceptorWS_PrepareReceptorOpalService_ws_nbcr_net_C')

        ## modifying MacroOutputNode dynamic ports
        output_Ports_2 = self.macroNetwork.opNode
        output_Ports_2.inputPorts[1].configure(singleConnection='auto')
        output_Ports_2.inputPorts[1].configure(name='AutodockVS_GetMainURLFromList_newurl')
        ## configure MacroNode input ports
        VirtualScreening_0.inputPorts[0].configure(name='PrepareReceptor_Pdb2pqrWS_CheckFileFormat_value')
        VirtualScreening_0.inputPorts[0].configure(datatype='receptor')
        VirtualScreening_0.inputPorts[1].configure(name='ComputeGrids_GetComputeGridsInputs_ligands')
        VirtualScreening_0.inputPorts[1].configure(datatype='LigandDB')
        VirtualScreening_0.inputPorts[2].configure(name='ComputeGrids_GetComputeGridsInputs_gpf_obj')
        VirtualScreening_0.inputPorts[2].configure(datatype='gpf_template')
        VirtualScreening_0.inputPorts[3].configure(name='AutodockVS_PrepareADVSInputs_dpf_template_obj')
        VirtualScreening_0.inputPorts[3].configure(datatype='dpf_template')
        VirtualScreening_0.inputPorts[4].configure(name='PrepareReceptor_PrepareReceptorWS_PrepareReceptorOpalService_ws_nbcr_net_C')
        VirtualScreening_0.inputPorts[4].configure(datatype='boolean')
        ## configure MacroNode output ports
        VirtualScreening_0.outputPorts[0].configure(name='AutodockVS_GetMainURLFromList_newurl')
        VirtualScreening_0.outputPorts[0].configure(datatype='string')

        VirtualScreening_0.shrink()

        ## reset modifications ##
        VirtualScreening_0.resetTags()
        VirtualScreening_0.buildOriginalList()



########################################################################
#
#    Vision Macro - Python source code - file generated by vision
#    Monday 28 June 2010 19:21:46 
#    
#       The Scripps Research Institute (TSRI)
#       Molecular Graphics Lab
#       La Jolla, CA 92037, USA
#
# Copyright: Daniel Stoffler, Michel Sanner and TSRI
#   
# revision: Guillaume Vareille
#  
#########################################################################
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/VisionInterface/Adt/Macro/PrepareReceptor.py,v 1.10 2010/07/01 23:42:45 jren Exp $
#
# $Id: PrepareReceptor.py,v 1.10 2010/07/01 23:42:45 jren Exp $
#

from NetworkEditor.macros import MacroNode
from NetworkEditor.macros import MacroNode
class PrepareReceptor(MacroNode):
    '''
    First runs PDB2PQR on the remote server, then runs Prepare Receptor on the remote server.
    PDB2PQR is skipped if the receptor object already contains info about a PQR or PDBQT  file.
    Prepare Receptor is skipped is a PDBQT file already exists on the local machine.

    Inputs:
    port 1: receptor object
    port 2: boolean value, the user should use CheckButton from Standard library to pass the value 
            to this port. 
            If checked, then Prepare Receptor will preserve all input charges ie. do not add new charges 
            (default is addition of gasteiger charges)

    Outputs:
    port 1: receptor_prepared object
    port 2: string containing location of the PDBQT file
 
    '''

    def __init__(self, constrkw={}, name='PrepareReceptor', **kw):
        kw['name'] = name
        apply( MacroNode.__init__, (self,), kw)

    def beforeAddingToNetwork(self, net):
        MacroNode.beforeAddingToNetwork(self, net)
        from WebServices.VisionInterface.WSNodes import wslib
        from Vision.StandardNodes import stdlib
        net.getEditor().addLibraryInstance(wslib,"WebServices.VisionInterface.WSNodes", "wslib")
        from WebServices.VisionInterface.WSNodes import addOpalServerAsCategory
        try:
            addOpalServerAsCategory("http://ws.nbcr.net/opal2", replace=False)
        except:
            pass

    def afterAddingToNetwork(self):
        masterNet = self.macroNetwork
        from NetworkEditor.macros import MacroNode
        MacroNode.afterAddingToNetwork(self)
        from WebServices.VisionInterface.WSNodes import wslib
        from Vision.StandardNodes import stdlib
        ## building macro network ##
        PrepareReceptor_0 = self
        from traceback import print_exc
        from WebServices.VisionInterface.WSNodes import wslib
        from Vision.StandardNodes import stdlib
        masterNet.getEditor().addLibraryInstance(wslib,"WebServices.VisionInterface.WSNodes", "wslib")
        from WebServices.VisionInterface.WSNodes import addOpalServerAsCategory
        try:
            addOpalServerAsCategory("http://ws.nbcr.net/opal2", replace=False)
        except:
            pass
        try:
            ## saving node input Ports ##
            input_Ports_1 = self.macroNetwork.ipNode
            apply(input_Ports_1.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
        except:
            print "WARNING: failed to restore MacroInputNode named input Ports in network self.macroNetwork"
            print_exc()
            input_Ports_1=None

        try:
            ## saving node output Ports ##
            output_Ports_2 = self.macroNetwork.opNode
            apply(output_Ports_2.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
        except:
            print "WARNING: failed to restore MacroOutputNode named output Ports in network self.macroNetwork"
            print_exc()
            output_Ports_2=None

        try:
            ## saving node Pdb2pqrWS ##
            from NetworkEditor.macros import MacroNode
            Pdb2pqrWS_3 = MacroNode(name='Pdb2pqrWS')
            self.macroNetwork.addNode(Pdb2pqrWS_3, 153, 86)
            input_Ports_4 = Pdb2pqrWS_3.macroNetwork.ipNode
            apply(input_Ports_4.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
            input_Ports_4.move(65, 10)
            output_Ports_5 = Pdb2pqrWS_3.macroNetwork.opNode
            apply(output_Ports_5.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
            output_Ports_5.move(65, 310)
            from Vision.StandardNodes import Generic
            CheckFileFormat_6 = Generic(constrkw={}, name='CheckFileFormat', library=stdlib)
            Pdb2pqrWS_3.macroNetwork.addNode(CheckFileFormat_6,82,62)
            apply(CheckFileFormat_6.addInputPort, (), {'singleConnection': True, 'name': 'value', 'cast': True, 'datatype': 'receptor', 'defaultValue': None, 'required': True, 'height': 8, 'width': 12, 'shape': 'triangle', 'color': '#99FF33', 'originalDatatype': 'None'})
            apply(CheckFileFormat_6.addOutputPort, (), {'name': 'receptor_obj', 'datatype': 'receptor', 'height': 8, 'width': 12, 'shape': 'triangle', 'color': '#99FF33'})
            apply(CheckFileFormat_6.addOutputPort, (), {'name': 'pqr_name', 'datatype': 'string', 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white'})
            apply(CheckFileFormat_6.addOutputPort, (), {'name': 'pdb_path', 'datatype': 'string', 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white'})
            code = """def doit(self, value):
    if value.get_ext_loc('''pqr''') != None or value.get_ext_loc('''pdbqt''') != None:
        self.outputData(receptor_obj=value)
        self.scheduleChildren(portList=[self.getOutputPortByName('''receptor_obj''')])
    else:
        pqr_name = value.get_id() + '''.pqr'''
        pdb_path = value.get_ext_loc('''pdb''')

        self.outputData(receptor_obj=value, pqr_name=pqr_name, pdb_path=pdb_path)
        self.scheduleChildren(portList=[self.getOutputPortByName('''receptor_obj'''), self.getOutputPortByName('''pqr_name'''), self.getOutputPortByName('''pdb_path''')])

"""
            CheckFileFormat_6.configure(function=code)
            apply(CheckFileFormat_6.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
            from NetworkEditor.items import FunctionNode
            Pdb2pqrOpalService_ws_nbcr_net_7 = FunctionNode(functionOrString='Pdb2pqrOpalService_ws_nbcr_net', host="http://ws.nbcr.net/opal2", namedArgs={'noopt': False, 'phi': False, 'psi': False, 'verbose': True, 'chain': False, 'nodebump': False, 'chi': False, 'output_file': '', 'ligand': '', 'hbond': False, 'inFile': '', 'with_ph': '', 'forcefield': 'AMBER', 'clean': False, 'inId': '', 'apbs_input': False, 'ffout': '', 'localRun': False, 'rama': False, 'execPath': '', 'assign_only': False}, constrkw={'functionOrString': "'Pdb2pqrOpalService_ws_nbcr_net'", 'host': '"http://ws.nbcr.net/opal2"', 'namedArgs': {'noopt': False, 'phi': False, 'psi': False, 'verbose': True, 'chain': False, 'nodebump': False, 'chi': False, 'output_file': '', 'ligand': '', 'hbond': False, 'inFile': '', 'with_ph': '', 'forcefield': 'AMBER', 'clean': False, 'inId': '', 'apbs_input': False, 'ffout': '', 'localRun': False, 'rama': False, 'execPath': '', 'assign_only': False}}, name='Pdb2pqrOpalService_ws_nbcr_net', library=wslib)
            Pdb2pqrWS_3.macroNetwork.addNode(Pdb2pqrOpalService_ws_nbcr_net_7,99,115)
            apply(Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['output_file'].configure, (), {'defaultValue': None, 'required': True})
            apply(Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['inFile'].configure, (), {'defaultValue': None, 'required': True})
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['noopt'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['phi'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['psi'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['verbose'].widget.set(1, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['chain'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['nodebump'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['chi'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['output_file'].rebindWidget()
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['output_file'].widget.set(r"", run=False)
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['output_file'].unbindWidget()
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['ligand'].widget.set(r"", run=False)
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['hbond'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['inFile'].rebindWidget()
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['inFile'].widget.set(r"", run=False)
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['inFile'].unbindWidget()
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['with_ph'].widget.set(r"", run=False)
            apply(Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['forcefield'].widget.configure, (), {'choices': ('AMBER', 'CHARMM', 'PARSE', 'TYL06')})
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['forcefield'].widget.set(r"AMBER", run=False)
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['clean'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['inId'].widget.set(r"", run=False)
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['apbs_input'].widget.set(0, run=False)
            apply(Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['ffout'].widget.configure, (), {'choices': ('AMBER', 'CHARMM', 'PARSE', 'TYL06')})
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['ffout'].widget.set(r"", run=False)
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['localRun'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['rama'].widget.set(0, run=False)
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['execPath'].widget.set(r"", run=False)
            Pdb2pqrOpalService_ws_nbcr_net_7.inputPortByName['assign_only'].widget.set(0, run=False)
            apply(Pdb2pqrOpalService_ws_nbcr_net_7.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
            from WebServices.VisionInterface.WSNodes import GetURLFromListNode
            GetURLFromList_8 = GetURLFromListNode(constrkw={}, name='GetURLFromList', library=wslib)
            Pdb2pqrWS_3.macroNetwork.addNode(GetURLFromList_8,117,169)
            GetURLFromList_8.inputPortByName['ext'].widget.set(r"pqr", run=False)
            apply(GetURLFromList_8.configure, (), {'paramPanelImmediate': 1})
            from Vision.StandardNodes import Generic
            UpdateReceptor_9 = Generic(constrkw={}, name='UpdateReceptor', library=stdlib)
            Pdb2pqrWS_3.macroNetwork.addNode(UpdateReceptor_9,82,256)
            apply(UpdateReceptor_9.addInputPort, (), {'singleConnection': True, 'name': 'receptor_obj', 'cast': True, 'datatype': 'receptor', 'defaultValue': None, 'required': True, 'height': 8, 'width': 12, 'shape': 'triangle', 'color': '#99FF33', 'originalDatatype': 'None'})
            apply(UpdateReceptor_9.addInputPort, (), {'singleConnection': True, 'name': 'pqr', 'cast': True, 'datatype': 'string', 'defaultValue': None, 'required': False, 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white', 'originalDatatype': 'None'})
            apply(UpdateReceptor_9.addOutputPort, (), {'name': 'receptor_obj', 'datatype': 'receptor', 'height': 8, 'width': 12, 'shape': 'triangle', 'color': '#99FF33'})
            apply(UpdateReceptor_9.addOutputPort, (), {'name': 'pdb2pqr_result', 'datatype': 'string', 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white'})
            code = """def doit(self, receptor_obj, pqr):
        if pqr != '''no data yet''':
            pdb2pqr_result = pqr
            receptor_obj.set_ext_loc(pqr)
        else:
            pdbqt = receptor_obj.get_ext_loc('''pdbqt''')
            pqr = receptor_obj.get_ext_loc('''pqr''')

            if pdbqt != None:
                pdb2pqr_result = pdbqt
            elif pqr != None:
                pdb2pqr_result = pqr

        pass
        self.outputData(receptor_obj=receptor_obj, pdb2pqr_result=pdb2pqr_result)





"""
            UpdateReceptor_9.configure(function=code)
            apply(UpdateReceptor_9.configure, (), {'paramPanelImmediate': 1, 'expanded': False})

            ## saving connections for network Pdb2pqrWS ##
            Pdb2pqrWS_3.macroNetwork.freeze()
            if CheckFileFormat_6 is not None and Pdb2pqrOpalService_ws_nbcr_net_7 is not None:
                try:
                    Pdb2pqrWS_3.macroNetwork.connectNodes(
                        CheckFileFormat_6, Pdb2pqrOpalService_ws_nbcr_net_7, "pqr_name", "output_file", blocking=True
                        , splitratio=[0.43278717885693868, 0.65281492247209427])
                except:
                    print "WARNING: failed to restore connection between CheckFileFormat_6 and Pdb2pqrOpalService_ws_nbcr_net_7 in network Pdb2pqrWS_3.macroNetwork"
            if CheckFileFormat_6 is not None and Pdb2pqrOpalService_ws_nbcr_net_7 is not None:
                try:
                    Pdb2pqrWS_3.macroNetwork.connectNodes(
                        CheckFileFormat_6, Pdb2pqrOpalService_ws_nbcr_net_7, "pdb_path", "inFile", blocking=True
                        , splitratio=[0.61978340673276544, 0.69977512898866912])
                except:
                    print "WARNING: failed to restore connection between CheckFileFormat_6 and Pdb2pqrOpalService_ws_nbcr_net_7 in network Pdb2pqrWS_3.macroNetwork"
            if Pdb2pqrOpalService_ws_nbcr_net_7 is not None and GetURLFromList_8 is not None:
                try:
                    Pdb2pqrWS_3.macroNetwork.connectNodes(
                        Pdb2pqrOpalService_ws_nbcr_net_7, GetURLFromList_8, "result", "urllist", blocking=True
                        , splitratio=[0.73247388879852338, 0.69014271703344965])
                except:
                    print "WARNING: failed to restore connection between Pdb2pqrOpalService_ws_nbcr_net_7 and GetURLFromList_8 in network Pdb2pqrWS_3.macroNetwork"
            if GetURLFromList_8 is not None and UpdateReceptor_9 is not None:
                try:
                    Pdb2pqrWS_3.macroNetwork.connectNodes(
                        GetURLFromList_8, UpdateReceptor_9, "url", "pqr", blocking=True
                        , splitratio=[0.35224914715245104, 0.47693375858097758])
                except:
                    print "WARNING: failed to restore connection between GetURLFromList_8 and UpdateReceptor_9 in network Pdb2pqrWS_3.macroNetwork"
            if CheckFileFormat_6 is not None and UpdateReceptor_9 is not None:
                try:
                    Pdb2pqrWS_3.macroNetwork.connectNodes(
                        CheckFileFormat_6, UpdateReceptor_9, "receptor_obj", "receptor_obj", blocking=True
                        , splitratio=[0.57865377076653579, 0.45824116584600394])
                except:
                    print "WARNING: failed to restore connection between CheckFileFormat_6 and UpdateReceptor_9 in network Pdb2pqrWS_3.macroNetwork"
            input_Ports_4 = Pdb2pqrWS_3.macroNetwork.ipNode
            if input_Ports_4 is not None and CheckFileFormat_6 is not None:
                try:
                    Pdb2pqrWS_3.macroNetwork.connectNodes(
                        input_Ports_4, CheckFileFormat_6, "new", "value", blocking=True
                        , splitratio=[0.28291581858817416, 0.70581213384962171])
                except:
                    print "WARNING: failed to restore connection between input_Ports_4 and CheckFileFormat_6 in network Pdb2pqrWS_3.macroNetwork"
            output_Ports_5 = Pdb2pqrWS_3.macroNetwork.opNode
            if UpdateReceptor_9 is not None and output_Ports_5 is not None:
                try:
                    Pdb2pqrWS_3.macroNetwork.connectNodes(
                        UpdateReceptor_9, output_Ports_5, "receptor_obj", "new", blocking=True
                        , splitratio=[0.24354423773729619, 0.72115604180615911])
                except:
                    print "WARNING: failed to restore connection between UpdateReceptor_9 and output_Ports_5 in network Pdb2pqrWS_3.macroNetwork"
            if UpdateReceptor_9 is not None and output_Ports_5 is not None:
                try:
                    Pdb2pqrWS_3.macroNetwork.connectNodes(
                        UpdateReceptor_9, output_Ports_5, "pdb2pqr_result", "new", blocking=True
                        , splitratio=[0.31141907324539131, 0.41794097665671487])
                except:
                    print "WARNING: failed to restore connection between UpdateReceptor_9 and output_Ports_5 in network Pdb2pqrWS_3.macroNetwork"
            Pdb2pqrWS_3.macroNetwork.unfreeze()

            ## modifying MacroInputNode dynamic ports
            input_Ports_4.outputPorts[1].configure(name='CheckFileFormat_value')

            ## modifying MacroOutputNode dynamic ports
            output_Ports_5.inputPorts[1].configure(singleConnection='auto')
            output_Ports_5.inputPorts[2].configure(singleConnection='auto')
            output_Ports_5.inputPorts[1].configure(name='UpdateReceptor_receptor_obj')
            output_Ports_5.inputPorts[2].configure(name='UpdateReceptor_pdb2pqr_result')
            Pdb2pqrWS_3.inputPorts[0].configure(name='CheckFileFormat_value')
            Pdb2pqrWS_3.inputPorts[0].configure(datatype='receptor')
            ## configure MacroNode input ports
            Pdb2pqrWS_3.outputPorts[0].configure(name='UpdateReceptor_receptor_obj')
            Pdb2pqrWS_3.outputPorts[0].configure(datatype='receptor')
            Pdb2pqrWS_3.outputPorts[1].configure(name='UpdateReceptor_pdb2pqr_result')
            Pdb2pqrWS_3.outputPorts[1].configure(datatype='string')
            ## configure MacroNode output ports
            Pdb2pqrWS_3.shrink()
            apply(Pdb2pqrWS_3.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
        except:
            print "WARNING: failed to restore MacroNode named Pdb2pqrWS in network self.macroNetwork"
            print_exc()
            Pdb2pqrWS_3=None

        try:
            ## saving node PrepareReceptorWS ##
            from NetworkEditor.macros import MacroNode
            PrepareReceptorWS_10 = MacroNode(name='PrepareReceptorWS')
            self.macroNetwork.addNode(PrepareReceptorWS_10, 217, 140)
            input_Ports_11 = PrepareReceptorWS_10.macroNetwork.ipNode
            apply(input_Ports_11.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
            input_Ports_11.move(145, 8)
            output_Ports_12 = PrepareReceptorWS_10.macroNetwork.opNode
            apply(output_Ports_12.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
            output_Ports_12.move(29, 412)
            from Vision.StandardNodes import Generic
            CheckFileFormat_13 = Generic(constrkw={}, name='CheckFileFormat', library=stdlib)
            PrepareReceptorWS_10.macroNetwork.addNode(CheckFileFormat_13,46,64)
            apply(CheckFileFormat_13.addInputPort, (), {'singleConnection': True, 'name': 'value', 'cast': True, 'datatype': 'receptor', 'defaultValue': None, 'required': True, 'height': 8, 'width': 12, 'shape': 'triangle', 'color': '#99FF33', 'originalDatatype': 'None'})
            apply(CheckFileFormat_13.addOutputPort, (), {'name': 'receptor_obj', 'datatype': 'receptor', 'height': 8, 'width': 12, 'shape': 'triangle', 'color': '#99FF33'})
            apply(CheckFileFormat_13.addOutputPort, (), {'name': 'receptor_url', 'datatype': 'string', 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white'})
            apply(CheckFileFormat_13.addOutputPort, (), {'name': 'receptor_local', 'datatype': 'string', 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white'})
            apply(CheckFileFormat_13.addOutputPort, (), {'name': 'download_loc', 'datatype': 'string', 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white'})
            code = """def doit(self, value):
    if value.get_ext_loc('''pdbqt''') != None:
        self.outputData(receptor_obj=value)
        self.scheduleChildren(portList=[self.getOutputPortByName('''receptor_obj''')])
    else:
        pqr = value.get_ext_loc('''pqr''')
        pdb = value.get_ext_loc('''pdb''')
        download_loc = value.get_workdir() + os.sep + value.get_id() + '''.pdbqt'''

        if pqr != None:
            if value.get_ext_type('''pqr''') == '''url''':
                receptor_url = pqr
                receptor_local = ""
            else:
                receptor_url = ""
                receptor_local = pqr
        elif pdb != None:
            if value.get_ext_type('''pdb''') == '''url''':
                receptor_url = pdb
                receptor_local = ""
            else:
                receptor_url = ""
                receptor_local = pdb
        else:
            print "*** ERROR: pqr and pqr both missing from receptor object"
            print receptor_obj.ext_loc
            return '''stop'''

        self.outputData(receptor_obj=value, receptor_url=receptor_url, receptor_local=receptor_local, download_loc=download_loc)
        self.scheduleChildren(portList=[self.getOutputPortByName('''receptor_obj'''), self.getOutputPortByName('''receptor_url'''), self.getOutputPortByName('''receptor_local'''), self.getOutputPortByName('''download_loc''')])





"""
            CheckFileFormat_13.configure(function=code)
            apply(CheckFileFormat_13.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
            from NetworkEditor.items import FunctionNode
            PrepareReceptorOpalService_ws_nbcr_net_14 = FunctionNode(functionOrString='PrepareReceptorOpalService_ws_nbcr_net', host="http://ws.nbcr.net/opal2", namedArgs={'C': False, 'url': '', 'o': '', 'r': '', 'v': False, 'localRun': False, 'execPath': ''}, constrkw={'functionOrString': "'PrepareReceptorOpalService_ws_nbcr_net'", 'host': '"http://ws.nbcr.net/opal2"', 'namedArgs': {'C': False, 'url': '', 'o': '', 'r': '', 'v': False, 'localRun': False, 'execPath': ''}}, name='PrepareReceptorOpalService_ws_nbcr_net', library=wslib)
            PrepareReceptorWS_10.macroNetwork.addNode(PrepareReceptorOpalService_ws_nbcr_net_14,180,128)
            apply(PrepareReceptorOpalService_ws_nbcr_net_14.inputPortByName['C'].configure, (), {'defaultValue': None})
            apply(PrepareReceptorOpalService_ws_nbcr_net_14.inputPortByName['url'].configure, (), {'defaultValue': None, 'required': True})
            apply(PrepareReceptorOpalService_ws_nbcr_net_14.inputPortByName['r'].configure, (), {'defaultValue': None, 'required': True})
            PrepareReceptorOpalService_ws_nbcr_net_14.inputPortByName['C'].rebindWidget()
            PrepareReceptorOpalService_ws_nbcr_net_14.inputPortByName['C'].widget.set(0, run=False)
            PrepareReceptorOpalService_ws_nbcr_net_14.inputPortByName['C'].unbindWidget()
            PrepareReceptorOpalService_ws_nbcr_net_14.inputPortByName['url'].rebindWidget()
            PrepareReceptorOpalService_ws_nbcr_net_14.inputPortByName['url'].widget.set(r"", run=False)
            PrepareReceptorOpalService_ws_nbcr_net_14.inputPortByName['url'].unbindWidget()
            PrepareReceptorOpalService_ws_nbcr_net_14.inputPortByName['o'].widget.set(r"", run=False)
            PrepareReceptorOpalService_ws_nbcr_net_14.inputPortByName['r'].rebindWidget()
            PrepareReceptorOpalService_ws_nbcr_net_14.inputPortByName['r'].widget.set(r"", run=False)
            PrepareReceptorOpalService_ws_nbcr_net_14.inputPortByName['r'].unbindWidget()
            PrepareReceptorOpalService_ws_nbcr_net_14.inputPortByName['v'].widget.set(0, run=False)
            PrepareReceptorOpalService_ws_nbcr_net_14.inputPortByName['localRun'].widget.set(0, run=False)
            PrepareReceptorOpalService_ws_nbcr_net_14.inputPortByName['execPath'].widget.set(r"", run=False)
            apply(PrepareReceptorOpalService_ws_nbcr_net_14.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
            from WebServices.VisionInterface.WSNodes import GetURLFromListNode
            GetURLFromList_15 = GetURLFromListNode(constrkw={}, name='GetURLFromList', library=wslib)
            PrepareReceptorWS_10.macroNetwork.addNode(GetURLFromList_15,180,186)
            GetURLFromList_15.inputPortByName['ext'].widget.set(r"pdbqt", run=False)
            apply(GetURLFromList_15.configure, (), {'paramPanelImmediate': 1})
            from WebServices.VisionInterface.WSNodes import DownloadToFileNode
            DownloadToFile_16 = DownloadToFileNode(constrkw={}, name='DownloadToFile', library=wslib)
            PrepareReceptorWS_10.macroNetwork.addNode(DownloadToFile_16,80,272)
            apply(DownloadToFile_16.inputPortByName['url'].configure, (), {'defaultValue': None})
            apply(DownloadToFile_16.inputPortByName['filename'].configure, (), {'defaultValue': None})
            DownloadToFile_16.inputPortByName['url'].rebindWidget()
            DownloadToFile_16.inputPortByName['url'].widget.set(r"", run=False)
            DownloadToFile_16.inputPortByName['url'].unbindWidget()
            DownloadToFile_16.inputPortByName['filename'].rebindWidget()
            DownloadToFile_16.inputPortByName['filename'].widget.set(r"", run=False)
            DownloadToFile_16.inputPortByName['filename'].unbindWidget()
            DownloadToFile_16.inputPortByName['overwrite'].widget.set(1, run=False)
            apply(DownloadToFile_16.configure, (), {'paramPanelImmediate': 1})
            from Vision.StandardNodes import Generic
            UpdateReceptor_17 = Generic(constrkw={}, name='UpdateReceptor', library=stdlib)
            PrepareReceptorWS_10.macroNetwork.addNode(UpdateReceptor_17,46,358)
            apply(UpdateReceptor_17.addInputPort, (), {'singleConnection': True, 'name': 'receptor_obj', 'cast': True, 'datatype': 'receptor', 'defaultValue': None, 'required': True, 'height': 8, 'width': 12, 'shape': 'triangle', 'color': '#99FF33', 'originalDatatype': 'None'})
            apply(UpdateReceptor_17.addInputPort, (), {'singleConnection': True, 'name': 'pdbqt', 'cast': True, 'datatype': 'string', 'defaultValue': None, 'required': False, 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white', 'originalDatatype': 'None'})
            apply(UpdateReceptor_17.addOutputPort, (), {'name': 'receptor_prepared_obj', 'datatype': 'receptor_prepared', 'height': 8, 'width': 12, 'shape': 'triangle', 'color': '#009900'})
            apply(UpdateReceptor_17.addOutputPort, (), {'name': 'receptor_result', 'datatype': 'string', 'height': 8, 'width': 12, 'shape': 'oval', 'color': 'white'})
            code = """def doit(self, receptor_obj, pdbqt):
        from AutoDockTools.VisionInterface.Adt.receptor_prepared import receptor_prepared
        if pdbqt != '''no data yet''':
            receptor_prepared_obj = receptor_prepared(pdbqt)
            receptor_obj.set_ext_loc(pdbqt)
            receptor_result = pdbqt
        else:
            pdbqt_org = receptor_obj.get_ext_loc('''pdbqt''')
            receptor_prepared_obj = receptor_prepared(pdbqt_org)
            receptor_result = pdbqt_org
        
        pass

        self.outputData(receptor_prepared_obj = receptor_prepared_obj, receptor_result=receptor_result)



"""
            UpdateReceptor_17.configure(function=code)
            apply(UpdateReceptor_17.configure, (), {'paramPanelImmediate': 1, 'expanded': False})

            ## saving connections for network PrepareReceptorWS ##
            PrepareReceptorWS_10.macroNetwork.freeze()
            input_Ports_11 = PrepareReceptorWS_10.macroNetwork.ipNode
            if input_Ports_11 is not None and CheckFileFormat_13 is not None:
                try:
                    PrepareReceptorWS_10.macroNetwork.connectNodes(
                        input_Ports_11, CheckFileFormat_13, "new", "value", blocking=True
                        , splitratio=[0.64907843562020084, 0.47057253960060474])
                except:
                    print "WARNING: failed to restore connection between input_Ports_11 and CheckFileFormat_13 in network PrepareReceptorWS_10.macroNetwork"
            if CheckFileFormat_13 is not None and PrepareReceptorOpalService_ws_nbcr_net_14 is not None:
                try:
                    PrepareReceptorWS_10.macroNetwork.connectNodes(
                        CheckFileFormat_13, PrepareReceptorOpalService_ws_nbcr_net_14, "receptor_url", "url", blocking=True
                        , splitratio=[0.6171192646814192, 0.42883431331137822])
                except:
                    print "WARNING: failed to restore connection between CheckFileFormat_13 and PrepareReceptorOpalService_ws_nbcr_net_14 in network PrepareReceptorWS_10.macroNetwork"
            if CheckFileFormat_13 is not None and PrepareReceptorOpalService_ws_nbcr_net_14 is not None:
                try:
                    PrepareReceptorWS_10.macroNetwork.connectNodes(
                        CheckFileFormat_13, PrepareReceptorOpalService_ws_nbcr_net_14, "receptor_local", "r", blocking=True
                        , splitratio=[0.66136144381723583, 0.28482114514549767])
                except:
                    print "WARNING: failed to restore connection between CheckFileFormat_13 and PrepareReceptorOpalService_ws_nbcr_net_14 in network PrepareReceptorWS_10.macroNetwork"
            if input_Ports_11 is not None and PrepareReceptorOpalService_ws_nbcr_net_14 is not None:
                try:
                    PrepareReceptorWS_10.macroNetwork.connectNodes(
                        input_Ports_11, PrepareReceptorOpalService_ws_nbcr_net_14, "new", "C", blocking=True
                        , splitratio=[0.53817543437956172, 0.61119702937572407])
                except:
                    print "WARNING: failed to restore connection between input_Ports_11 and PrepareReceptorOpalService_ws_nbcr_net_14 in network PrepareReceptorWS_10.macroNetwork"
            if PrepareReceptorOpalService_ws_nbcr_net_14 is not None and GetURLFromList_15 is not None:
                try:
                    PrepareReceptorWS_10.macroNetwork.connectNodes(
                        PrepareReceptorOpalService_ws_nbcr_net_14, GetURLFromList_15, "result", "urllist", blocking=True
                        , splitratio=[0.43369347916867251, 0.26566786648658042])
                except:
                    print "WARNING: failed to restore connection between PrepareReceptorOpalService_ws_nbcr_net_14 and GetURLFromList_15 in network PrepareReceptorWS_10.macroNetwork"
            if GetURLFromList_15 is not None and DownloadToFile_16 is not None:
                try:
                    PrepareReceptorWS_10.macroNetwork.connectNodes(
                        GetURLFromList_15, DownloadToFile_16, "url", "url", blocking=True
                        , splitratio=[0.70502483723033316, 0.71683158018466053])
                except:
                    print "WARNING: failed to restore connection between GetURLFromList_15 and DownloadToFile_16 in network PrepareReceptorWS_10.macroNetwork"
            if CheckFileFormat_13 is not None and DownloadToFile_16 is not None:
                try:
                    PrepareReceptorWS_10.macroNetwork.connectNodes(
                        CheckFileFormat_13, DownloadToFile_16, "download_loc", "filename", blocking=True
                        , splitratio=[0.60455581032885575, 0.35606890231085681])
                except:
                    print "WARNING: failed to restore connection between CheckFileFormat_13 and DownloadToFile_16 in network PrepareReceptorWS_10.macroNetwork"
            if CheckFileFormat_13 is not None and UpdateReceptor_17 is not None:
                try:
                    PrepareReceptorWS_10.macroNetwork.connectNodes(
                        CheckFileFormat_13, UpdateReceptor_17, "receptor_obj", "receptor_obj", blocking=True
                        , splitratio=[0.74908230670336207, 0.38226563370205602])
                except:
                    print "WARNING: failed to restore connection between CheckFileFormat_13 and UpdateReceptor_17 in network PrepareReceptorWS_10.macroNetwork"
            if DownloadToFile_16 is not None and UpdateReceptor_17 is not None:
                try:
                    PrepareReceptorWS_10.macroNetwork.connectNodes(
                        DownloadToFile_16, UpdateReceptor_17, "filename", "pdbqt", blocking=True
                        , splitratio=[0.38698393490575966, 0.2874426709930199])
                except:
                    print "WARNING: failed to restore connection between DownloadToFile_16 and UpdateReceptor_17 in network PrepareReceptorWS_10.macroNetwork"
            output_Ports_12 = PrepareReceptorWS_10.macroNetwork.opNode
            if UpdateReceptor_17 is not None and output_Ports_12 is not None:
                try:
                    PrepareReceptorWS_10.macroNetwork.connectNodes(
                        UpdateReceptor_17, output_Ports_12, "receptor_prepared_obj", "new", blocking=True
                        , splitratio=[0.69589857263374943, 0.59566078506240416])
                except:
                    print "WARNING: failed to restore connection between UpdateReceptor_17 and output_Ports_12 in network PrepareReceptorWS_10.macroNetwork"
            if UpdateReceptor_17 is not None and output_Ports_12 is not None:
                try:
                    PrepareReceptorWS_10.macroNetwork.connectNodes(
                        UpdateReceptor_17, output_Ports_12, "receptor_result", "new", blocking=True
                        , splitratio=[0.44926362052272689, 0.38628711158860896])
                except:
                    print "WARNING: failed to restore connection between UpdateReceptor_17 and output_Ports_12 in network PrepareReceptorWS_10.macroNetwork"
            PrepareReceptorWS_10.macroNetwork.unfreeze()

            ## modifying MacroInputNode dynamic ports
            input_Ports_11.outputPorts[1].configure(name='CheckFileFormat_value')
            input_Ports_11.outputPorts[2].configure(name='PrepareReceptorOpalService_ws_nbcr_net_C')

            ## modifying MacroOutputNode dynamic ports
            output_Ports_12.inputPorts[1].configure(singleConnection='auto')
            output_Ports_12.inputPorts[2].configure(singleConnection='auto')
            output_Ports_12.inputPorts[1].configure(name='UpdateReceptor_receptor_prepared_obj')
            output_Ports_12.inputPorts[2].configure(name='UpdateReceptor_receptor_result')
            PrepareReceptorWS_10.inputPorts[0].configure(name='CheckFileFormat_value')
            PrepareReceptorWS_10.inputPorts[0].configure(datatype='receptor')
            PrepareReceptorWS_10.inputPorts[1].configure(name='PrepareReceptorOpalService_ws_nbcr_net_C')
            PrepareReceptorWS_10.inputPorts[1].configure(datatype='boolean')
            ## configure MacroNode input ports
            PrepareReceptorWS_10.outputPorts[0].configure(name='UpdateReceptor_receptor_prepared_obj')
            PrepareReceptorWS_10.outputPorts[0].configure(datatype='receptor_prepared')
            PrepareReceptorWS_10.outputPorts[1].configure(name='UpdateReceptor_receptor_result')
            PrepareReceptorWS_10.outputPorts[1].configure(datatype='string')
            ## configure MacroNode output ports
            PrepareReceptorWS_10.shrink()
            apply(PrepareReceptorWS_10.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
        except:
            print "WARNING: failed to restore MacroNode named PrepareReceptorWS in network self.macroNetwork"
            print_exc()
            PrepareReceptorWS_10=None

        #self.macroNetwork.run()
        self.macroNetwork.freeze()

        ## saving connections for network PrepareReceptor ##
        input_Ports_1 = self.macroNetwork.ipNode
        Pdb2pqrWS_3 = self.macroNetwork.nodes[2]
        if input_Ports_1 is not None and Pdb2pqrWS_3 is not None:
            try:
                self.macroNetwork.connectNodes(
                    input_Ports_1, Pdb2pqrWS_3, "new", "CheckFileFormat_value", blocking=True
                    , splitratio=[0.73531020066010289, 0.24042404972613104])
            except:
                print "WARNING: failed to restore connection between input_Ports_1 and Pdb2pqrWS_3 in network self.macroNetwork"
        PrepareReceptorWS_10 = self.macroNetwork.nodes[3]
        if Pdb2pqrWS_3 is not None and PrepareReceptorWS_10 is not None:
            try:
                self.macroNetwork.connectNodes(
                    Pdb2pqrWS_3, PrepareReceptorWS_10, "UpdateReceptor_receptor_obj", "CheckFileFormat_value", blocking=True
                    , splitratio=[0.54479169827535734, 0.4847384502006486])
            except:
                print "WARNING: failed to restore connection between Pdb2pqrWS_3 and PrepareReceptorWS_10 in network self.macroNetwork"
        if input_Ports_1 is not None and PrepareReceptorWS_10 is not None:
            try:
                self.macroNetwork.connectNodes(
                    input_Ports_1, PrepareReceptorWS_10, "new", "PrepareReceptorOpalService_ws_nbcr_net_C", blocking=True
                    , splitratio=[0.37730151685222141, 0.48897820190498614])
            except:
                print "WARNING: failed to restore connection between input_Ports_1 and PrepareReceptorWS_10 in network self.macroNetwork"
        output_Ports_2 = self.macroNetwork.opNode
        if PrepareReceptorWS_10 is not None and output_Ports_2 is not None:
            try:
                self.macroNetwork.connectNodes(
                    PrepareReceptorWS_10, output_Ports_2, "UpdateReceptor_receptor_prepared_obj", "new", blocking=True
                    , splitratio=[0.64110508898616414, 0.45535290355389196])
            except:
                print "WARNING: failed to restore connection between PrepareReceptorWS_10 and output_Ports_2 in network self.macroNetwork"
        if PrepareReceptorWS_10 is not None and output_Ports_2 is not None:
            try:
                self.macroNetwork.connectNodes(
                    PrepareReceptorWS_10, output_Ports_2, "UpdateReceptor_receptor_result", "new", blocking=True
                    , splitratio=[0.45846842546122241, 0.44240009354017412])
            except:
                print "WARNING: failed to restore connection between PrepareReceptorWS_10 and output_Ports_2 in network self.macroNetwork"
        self.macroNetwork.runOnNewData.value = False

        ## modifying MacroInputNode dynamic ports
        input_Ports_1 = self.macroNetwork.ipNode
        input_Ports_1.outputPorts[1].configure(name='Pdb2pqrWS_CheckFileFormat_value')
        input_Ports_1.outputPorts[2].configure(name='PrepareReceptorWS_PrepareReceptorOpalService_ws_nbcr_net_C')

        ## modifying MacroOutputNode dynamic ports
        output_Ports_2 = self.macroNetwork.opNode
        output_Ports_2.inputPorts[1].configure(singleConnection='auto')
        output_Ports_2.inputPorts[2].configure(singleConnection='auto')
        output_Ports_2.inputPorts[1].configure(name='PrepareReceptorWS_UpdateReceptor_receptor_prepared_obj')
        output_Ports_2.inputPorts[2].configure(name='PrepareReceptorWS_UpdateReceptor_receptor_result')
        ## configure MacroNode input ports
        PrepareReceptor_0.inputPorts[0].configure(name='Pdb2pqrWS_CheckFileFormat_value')
        PrepareReceptor_0.inputPorts[0].configure(datatype='receptor')
        PrepareReceptor_0.inputPorts[1].configure(name='PrepareReceptorWS_PrepareReceptorOpalService_ws_nbcr_net_C')
        PrepareReceptor_0.inputPorts[1].configure(datatype='boolean')
        ## configure MacroNode output ports
        PrepareReceptor_0.outputPorts[0].configure(name='PrepareReceptorWS_UpdateReceptor_receptor_prepared_obj')
        PrepareReceptor_0.outputPorts[0].configure(datatype='receptor_prepared')
        PrepareReceptor_0.outputPorts[1].configure(name='PrepareReceptorWS_UpdateReceptor_receptor_result')
        PrepareReceptor_0.outputPorts[1].configure(datatype='string')

        PrepareReceptor_0.shrink()

        ## reset modifications ##
        PrepareReceptor_0.resetTags()
        PrepareReceptor_0.buildOriginalList()



#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/write_largest_cluster_ligand.py,v 1.3 2008/05/28 15:40:15 rhuey Exp $
#
import os, glob

from MolKit import Read
from AutoDockTools.Docking import Docking
from mglutil.math.rmsd import RMSDCalculator




if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        print "Usage: write_largest_cluster_ligand.py "
        print "    This script does the following: "
        print "         (1) read all the files with extension '.dlg' into one Docking"
        print "         (2) compute a clustering at the specified rms tolerance "
        print "         (3) write the ligand with the coordinates of the "
        print "             lowest-energy conformation in the largest cluster to a file" 
        print "    "
        print "    Optional parameters:"
        print "        [-t]    rms_tolerance (default 1.5)"
        print "        [-o pdbqt_filename] (default ligandstem_BC.pdbqt)"
        print "        [-v]    verbose output"

    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 't:o:vh')
    except getopt.GetoptError, msg:
        print 'write_largest_cluster_ligand.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    # optional parameters
    verbose = None
    #-o outputfilename
    outputfilename = None
    #-t rms_tolerance
    rms_tolerance = 1.5

    #'t:o:vh'
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-o', '--o'):
            outputfilename = a
            if verbose: print 'set outputfilename to ', a
        if o in ('-t', '--t'):
            rms_tolerance = float(a)
            if verbose: print 'set rms_tolerance to ', a
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()

    dlg_list = glob.glob('*.dlg')
    d = Docking()
    for dlg in dlg_list:
        d.readDlg(dlg)
    d.clusterer.rmsTool = RMSDCalculator(d.ligMol.allAtoms.coords[:])
    d.clusterer.make_clustering(rms_tolerance)
    clustering = d.clusterer.clustering_dict[rms_tolerance]
    largest = clustering[0]
    for clust in clustering:
        if len(clust)>len(largest):
            largest = clust
    #update the coordinates with those of conf with lowest energy in this largest cluster
    d.ch.set_conformation(largest[0])
    parser = d.ligMol.parser
    lines = []
    #have to add newline character to lines read from dlg
    for l in parser.allLines:
        l+= '\n'
        lines.append(l)
    parser.allLines = lines
    coords = d.ligMol.allAtoms.coords
    if outputfilename is None:
        outputfilename = d.ligMol.name  + '_BC.pdbqt'
    parser.write_with_new_coords(coords, outputfilename) 
    if verbose:
        print 'wrote %s' %outputfilename


# To execute this command type:
# write_largest_cluster_ligand.py  [-t rms_tolerance, -o outputfilename] -v







#!/usr/bin/env python
# $Id: superimpose_based_on_subset.py,v 1.2 2007/12/04 15:03:24 rhuey Exp $

#---------------------------------------------------------------------------#
#   Authors: Ruth Huey, Sandro Cosconati                                    #
#   Copyright: Ruth Huey, Sandro Cosconati  TSRI 2007                       #
#   usage:                                                                  #
#      superimpose_based_on_subsetfile.py -r reffile -m mobfile             #
#               -f subsetfile [-o outputfile -g mobsubsetfile -v verbose]   #
#                                                                           # 
#   subsetfile syntax: each line a single range of residues (zero-based)    #
#       15-17  #use residues at position 15 + 16                            #
#       67-68  #use residue at position  67                                 #
#---------------------------------------------------------------------------#


import os
from MolKit import Read
from MolKit.molecule import AtomSet
from mglutil.math.rigidFit import RigidfitBodyAligner


def get_atoms(mol, list_of_indicies, names_to_use=['N','CA','C'], verbose=False):
    if verbose: 
        print "in get_atoms with list of indicies:" 
        print list_of_indicies
    if not len(list_of_indicies):
        raise 'invalid input: list of indicies is empty!'
    atoms = AtomSet()
    num_res = 0
    for item in list_of_indicies:
        first, second = item
        #check for valid index and for end of chain
        max_index = len(mol.chains.residues)-1
        assert first<=max_index, 'invalid start of residue range'
        assert second<=max_index, 'invalid end of residue range'
        assert second>=first, 'second index cannot be smaller than first'
        if second==max_index:
            #ie mol.chains.residues[second]==mol.chains.residues[-1]:
            these_res = mol.chains.residues[first:]
        else:
            these_res = mol.chains.residues[first:second+1]
        if verbose: print "Adding  %3d residues " %(len(these_res)),
        num_res+=len(these_res)
        if verbose: print "Now there are %d residues total" %(num_res)
        for r in these_res:
            for n in names_to_use:
                atoms.append( r.atoms.get(n)[0])
    assert len(atoms), 'invalid input: lists of indicies did not correspond to any residues!'
    if verbose: print 'returning %d atoms' %(len(atoms))
    return atoms



if __name__ == "__main__":
    import sys
    import getopt

    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: superimpose_based_on_subset_file.py -r reffile -m mobfile"
        print " Description of  command..."
        print "       -r reference filename"
        print "       -m moving filename"
        print "       -f name of file listing reference residue ranges: "
        print "            one range per line such as:\n26-26\n35-39\n"
        print " Optional parameters:"
        print "      [-o] output filename (default is stem of moving file + '_superimposed.pdb')"
        print "      [-g] name of file listing mobile residue ranges, one range per line eg 32-36"
        print "            use this option if the numbering of the residues differs  "
        print "      [-x] string_for_atoms_to_use listing which atoms in each residue in ranges of "
        print "           residues read from file to use for superimposition"
        print "               (default is N_CA_C)"
        print "      [-v] verbose output"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'r:m:f:o:x:g:vh')
    except getopt.GetoptError, msg:
        print "superimpose_based_on_subsetfile.py: %s" %msg
        usage()
        sys.exit(2)

    # initialize the required parameters
    #  -r:    filename for reference molecule
    reference_filename = None
    #  -m:    filename for moving molecule
    moving_filename = None
    #  -f:    filename for reference residue ranges
    refrange_filename = None
    #  -g:    filename for mobile residue ranges
    mobrange_filename = None
    #  -o:  outputfilename for moving molecule with new coords
    outputfilename = None

    #initialize the optional parameters
    verbose = None
    string_for_atoms_to_use = None
    mobrange_filename = None

    #r:m:f:o:x:vh
    for o, a in opt_list:
       if o in ('-r', '-r'):
           reference_filename = a
       if o in ('-m','-m'):
           moving_filename = a
       if o in ('-f','-f'):
           refrange_filename = a
       if o in ('-g', '-g'):
           mobrange_filename = a
       if o in ('-x', '-x'):
           string_for_atoms_to_use = a
       if o in ('-o', '-o'):
           outputfilename = a
       if o in ('-v', '-v'):
           verbose = True


    if not reference_filename:
        print "superimpose_based_on_subsetfile: reference filename must be specified"
        usage()
        sys.exit()
    if not moving_filename:
        print "superimpose_based_on_subsetfile: moving filename must be specified"
        usage()
        sys.exit()
    if not refrange_filename:
        print "superimpose_based_on_subsetfile: refrange_filename filename must be specified"
        usage()
        sys.exit()
     
    from MolKit import Read       #   Read always returns a MoleculeSet 
    refMol = Read(reference_filename)[0]  # the first Molecule in the set created 
    mobMol = Read(moving_filename)[0] # the first Molecule in the set created from 1hsg.pdb
    fptr = open(refrange_filename)
    #12-20\n36-40\n155-162\n175-178\n233-238\n338-345\n
    reflist = []
    lines = fptr.readlines()
    fptr.close()
    for l in lines:
        #sample line "12-20\n"
        #handle blank lines gracefully
        l = l.strip()
        if len(l):
            first, second  = map(int, l.split('-'))
            reflist.append((first,second))
    if verbose: print "reflist=", reflist
    moblist = []
    if mobrange_filename is not None:
        fptr = open(mobrange_filename)
        lines = fptr.readlines()
        fptr.close()
        for l in lines:
            #handle blank lines gracefully
            l = l.strip()
            if len(l):
                first,second  = map(int, l.split('-'))
                moblist.append((first,second))
        if verbose: print "moblist=", moblist
    else:
        moblist = reflist[:]
    #refMol.chains.residues.atoms       # all levels of hierarchical data structure
    #refMol.allAtoms                    # short cut to lowest level....
    #mobMol.allAtoms                    #each atom has 'coords' attribute
    from mglutil.math.rigidFit import RigidfitBodyAligner
    rigidfitAligner = RigidfitBodyAligner()
    if  string_for_atoms_to_use is None:
        string_for_atoms_to_use = "N_CA_C"
    names_to_use = string_for_atoms_to_use.split('_')
    refAtoms = get_atoms(refMol, reflist, names_to_use, verbose)
    refCoords = refAtoms.coords #or refMol.chains.residues.atoms.get('backbone').coords
    #check that there are some
    if len(refCoords)==0:
        print "NO REF COORDS!!!"
        raise 'Unable to superimpose because no reference coordinates were specified!!!' 
    mobAtoms = get_atoms(mobMol, moblist, names_to_use, verbose)
    mobCoords = mobAtoms.coords
    if len(mobCoords)==0:
        print "NO MOB COORDS!!!"
        raise 'Unable to superimpose because no mobile coordinates were specified!!!' 
    #check that moblist and reflist are of the same length    
    assert len(refCoords)==len(mobCoords)

    rigidfitAligner.setRefCoords(refCoords)  
    rigidfitAligner.rigidFit(mobCoords)
    #now rigidfitAligner has attributes 'rotationMatrix' and 'translationMatrix'

    newCoords = rigidfitAligner.transformCoords(mobMol.allAtoms.coords)
    #make sure the coordinates have the correct shape, ie (num_atoms, 3)
    if not newCoords is None and newCoords.shape[1]!=3:
        newCoords=newCoords[:,:3]   
    #newCoords is 2D Numeric array with dimensions  num_atoms, and length of each coordinate
    #if verbose: print "newCoords[0]=", newCoords[0]
    # end by writing a file with the transformed coordinates:
    #the parser of the moving molecule knows about all the input lines
    # so use it to write a file with the new coords:
    if outputfilename is None:
        EXT = os.path.splitext(refMol.parser.filename)[1]
        outputfilename = refMol.name + "_superimposed"+ EXT
    mobMol.parser.write_with_new_coords(newCoords, outputfilename)
    if verbose: print "wrote ", outputfilename

# To execute this command type:
# superimpose_based_on_subset_file.py -r reffile -m mobfile -o outfile [-x  string_for_atoms_to_use] [-v]
  

